from django.db import models


class Test(models.Model):
    name = models.CharField(max_length=100)


class Root(models.Model):
    name = models.CharField(max_length=100)


class Child(models.Model):
    name = models.CharField(max_length=100)
    odd = models.IntegerField(default=0)
    root = models.ForeignKey(Root)


class Custom(models.Model):
    name = models.CharField(max_length=100)



""" ADREST inclusion tags. """
from django.template import Library, VariableDoesNotExist
from django.template.base import TagHelperNode, parse_bits
from django.template.loader import get_template


register = Library()

# Fix django templatetags module loader
__path__ = ""


class AdrestInclusionNode(TagHelperNode):

    """ Service class for tags. """

    def render(self, context):
        """ Render node.

        :return str: Rendered string.

        """
        try:
            args, ctx = self.get_resolved_arguments(context)
            target = args[0]
            if not target:
                return ''
            ctx['content'] = target
        except VariableDoesNotExist:
            return ''

        emitter = context.get('emitter')
        t_name = emitter.get_template_path(target)
        t = get_template(t_name)
        context.dicts.append(ctx)
        response = t.nodelist.render(context)
        context.pop()
        return response


def adrest_include(parser, token):
    """ Include adrest_template for any objects.

    :return str: Rendered string.

    """
    bits = token.split_contents()[1:]
    args, kwargs = parse_bits(
        parser, bits, ['content'], 'args', 'kwargs', tuple(),
        False, 'adrest_include')
    return AdrestInclusionNode(False, args, kwargs)
adrest_include = register.tag(adrest_include)


def adrest_jsonify(content, **options):
    """ Serialize any object to JSON .

    :return str: Rendered string.

    """
    from adrest.utils.serializer import JSONSerializer
    worker = JSONSerializer(**options)
    return worker.serialize(content)
adrest_jsonify = register.simple_tag(adrest_jsonify)



from django.core.exceptions import ValidationError

from .status import HTTP_400_BAD_REQUEST


class HttpError(Exception):
    " Represents HTTP Error. "

    def __init__(self, content, status=HTTP_400_BAD_REQUEST, emitter=None):
        self.content, self.status, self.emitter = content, status, emitter
        super(HttpError, self).__init__(content)

    def __str__(self):
        return self.content

    __repr__ = __str__


class FormError(ValidationError):
    " Represents Form Error. "

    def __init__(self, form, emitter=None):
        self.form, self.emitter = form, emitter
        super(FormError, self).__init__(form.errors.as_text())



from django.test import RequestFactory
from django.views.generic import View
from mixer.backend.django import mixer

from ..api import api as API
from adrest.mixin import DynamicMixin


class CoreDynamicTest(API.testCase):

    def test_base(self):

        pirates = mixer.cycle(3).blend('core.pirate')

        class SomeResource(DynamicMixin, View):

            class Meta:
                model = 'core.pirate'

            def dispatch(self, request, **resources):
                return self.get_collection(request, **resources)

        rf = RequestFactory()
        resource = SomeResource()

        response = resource.dispatch(rf.get('/'))
        self.assertEqual(len(response), len(pirates))

        response = resource.dispatch(rf.get('/?name=' + pirates[0].name))
        self.assertEqual(list(response), [pirates[0]])

        response = resource.dispatch(
            rf.get('/?adr-sort=name&adr-sort=captain'))
        self.assertEqual(list(response), sorted(
            pirates, key=lambda p: (p.name, p.captain)))

    def test_pagination(self):

        pirates = mixer.cycle(3).blend('core.pirate')

        class SomeResource(DynamicMixin, View):

            class Meta:
                model = 'core.pirate'
                limit_per_page = 2

            def dispatch(self, request, **resources):
                collection = self.get_collection(request, **resources)
                return self.paginate(request, collection)

        rf = RequestFactory()
        resource = SomeResource()

        response = resource.dispatch(rf.get('/'))
        self.assertEqual(len(response.resources), 2)

        resource._meta.limit_per_page = 0
        response = resource.dispatch(rf.get('/'))
        self.assertEqual(len(response), len(pirates))

        response = resource.dispatch(rf.get('/?adr-max=1'))
        self.assertEqual(len(response.resources), 1)

# lint_ignore=C0110,E1103



""" ADREST related models. """

from django.db import models
from django.utils.encoding import smart_unicode

from . import settings
from .signals import api_request_finished


# Preloads ADREST tags
try:
    from django.template.base import builtins
    from .templatetags import register

    builtins.append(register)

except ImportError:
    pass


# Access log
# -----------
if settings.ADREST_ACCESS_LOG:

    class Access(models.Model):

        """ Log api queries. """

        created_at = models.DateTimeField(auto_now_add=True)
        uri = models.CharField(max_length=100)
        status_code = models.PositiveIntegerField()
        version = models.CharField(max_length=25)
        method = models.CharField(max_length=10, choices=(
            ('GET', 'GET'),
            ('POST', 'POST'),
            ('PUT', 'PUT'),
            ('DELETE', 'DELETE'),
            ('OPTIONS', 'OPTIONS'),
        ))
        request = models.TextField()
        response = models.TextField()
        identifier = models.CharField(max_length=255)

        class Meta():
            ordering = ["-created_at"]
            verbose_name_plural = "Access"

        def __unicode__(self):
            return "#{0} {1}:{2}:{3}".format(
                self.pk, self.method, self.status_code, self.uri)

    def save_log(sender, response=None, request=None, **resources):
        """ Save log to db. """
        resource = sender

        if not resource._meta.log:
            return

        try:
            content = smart_unicode(response.content)[:5000]
        except (UnicodeDecodeError, UnicodeEncodeError):
            if response and response['Content-Type'].lower() not in \
                    [emitter.media_type.lower()
                     for emitter in resource.emitters]:
                content = 'Invalid response content encoding'
            else:
                content = response.content[:5000]

        Access.objects.create(
            uri=request.path_info,
            method=request.method,
            version=str(resource.api or ''),
            status_code=response.status_code,
            request='%s\n\n%s' % (str(request.META), str(
                getattr(request, 'data', ''))),
            identifier=resource.identifier or request.META.get(
                'REMOTE_ADDR', 'anonymous'),
            response=content)

    api_request_finished.connect(save_log)


# Access keys
# -----------
if settings.ADREST_ACCESSKEY:

    import uuid
    from django.conf import settings as django_settings

    class AccessKey(models.Model):

        """ API key. """

        key = models.CharField(max_length=40, blank=True)
        user = models.ForeignKey(django_settings.AUTH_USER_MODEL)
        created = models.DateTimeField(auto_now_add=True)

        class Meta():
            ordering = ["-created"]
            unique_together = 'user', 'key'

        def __unicode__(self):
            return u'#%s %s "%s"' % (self.pk, self.user, self.key)

        def save(self, **kwargs):
            self.key = self.key or str(uuid.uuid4()).replace('-', '')
            super(AccessKey, self).save(**kwargs)

    # Auto create key for created user
    def create_api_key(sender, created=False, instance=None, **kwargs):
        """ Create key for user. """
        if created and instance:
            AccessKey.objects.create(user=instance)

    # Connect create handler to user save event
    if settings.ADREST_AUTO_CREATE_ACCESSKEY:
        from django import VERSION

        user_model = django_settings.AUTH_USER_MODEL
        if VERSION < (1, 7):
            from django.contrib.auth import get_user_model
            user_model = get_user_model()

        models.signals.post_save.connect(create_api_key, sender=user_model)


# pylama:ignore=E1002



import abc
import json as js

from .exceptions import HttpError
from .status import HTTP_400_BAD_REQUEST
from .tools import FrozenDict


__all__ = 'FormParser', 'JSONParser', 'RawParser', 'XMLParser', 'AbstractParser'


class AbstractParser(object):
    " Base class for parsers. "

    media_type = None

    __meta__ = abc.ABCMeta

    def __init__(self, resource):
        self.resource = resource

    @abc.abstractmethod
    def parse(self, request):
        raise NotImplementedError


class RawParser(AbstractParser):
    " Return raw post data. "

    media_type = 'text/plain'

    @staticmethod
    def parse(request):
        return request.body


class FormParser(AbstractParser):
    " Parse user data from form data. "

    media_type = 'application/x-www-form-urlencoded'

    @staticmethod
    def parse(request):
        return FrozenDict((k, v if len(v) > 1 else v[0])
                          for k, v in request.POST.iterlists())


class JSONParser(AbstractParser):
    """ Parse user data from JSON.
        http://en.wikipedia.org/wiki/JSON
    """

    media_type = 'application/json'

    @staticmethod
    def parse(request):
        try:
            return js.loads(request.body)
        except ValueError, e:
            raise HttpError('JSON parse error - %s'.format(e),
                            status=HTTP_400_BAD_REQUEST)


class XMLParser(RawParser):
    " Parse user data from XML. "

    media_type = 'application/xml'



import hashlib
import logging
import re

from django.core.cache import cache
from django.db.models import Model, get_model
from django.db.models.base import ModelBase
from django.db.models.query import QuerySet
from django.utils.encoding import smart_str


LOG = logging.getLogger('cache')


def cached_instance(model, timeout=None, **filters):
    """ Auto cached model instance.
    """
    if isinstance(model, basestring):
        model = _str_to_model(model)

    cache_key = generate_cache_key(model, **filters)
    return get_cached(cache_key, model.objects.select_related().get, kwargs=filters)


def cached_query(qs, timeout=None):
    """ Auto cached queryset and generate results.
    """
    cache_key = generate_cache_key(qs)
    return get_cached(cache_key, list, args=(qs,), timeout=None)


def clean_cache(cached, **kwargs):
    " Generate cache key and clean cached value. "

    if isinstance(cached, basestring):
        cached = _str_to_model(cached)

    cache_key = generate_cache_key(cached, **kwargs)
    cache.delete(cache_key)


def generate_cache_key(cached, **kwargs):
    """ Auto generate cache key for model or queryset
    """

    if isinstance(cached, QuerySet):
        key = str(cached.query)

    elif isinstance(cached, (Model, ModelBase)):
        key = '%s.%s:%s' % (cached._meta.app_label,
                            cached._meta.module_name,
                            ','.join('%s=%s' % item for item in kwargs.iteritems()))

    else:
        raise AttributeError("Objects must be queryset or model.")

    if not key:
        raise Exception('Cache key cannot be empty.')

    key = clean_cache_key(key)
    return key


def clean_cache_key(key):
    """ Replace spaces with '-' and hash if length is greater than 250.
    """
    cache_key = re.sub(r'\s+', '-', key)
    cache_key = smart_str(cache_key)

    if len(cache_key) > 200:
        cache_key = cache_key[:150] + '-' + hashlib.md5(cache_key).hexdigest()

    return cache_key


def get_cached(cache_key, func, timeout=None, args=None, kwargs=None):
    args = args or list()
    kwargs = kwargs or dict()
    result = cache.get(cache_key)

    if result is None:

        if timeout is None:
            timeout = cache.default_timeout

        result = func(*args, **kwargs)
        cache.set(cache_key, result, timeout=timeout)

    return result


def _str_to_model(string):
    assert '.' in string, ("'model_class' must be either a model"
                           " or a model name in the format"
                           " app_label.model_name")
    app_label, model_name = string.split(".")
    return get_model(app_label, model_name)



from django.db import models


class Author(models.Model):
    name = models.CharField(max_length=200, help_text='Name of author')


class Book(models.Model):
    name = models.CharField(max_length=200, help_text='Name of book')
    author = models.ForeignKey(Author)



""" ADRest serializers. """
import collections
import inspect
from numbers import Number
from datetime import datetime, date, time
from decimal import Decimal

from django.db.models import Model, Manager
from django.utils.encoding import smart_unicode
import json as js

from .tools import as_tuple


__all__ = 'JSONSerializer', 'XMLSerializer', 'BaseSerializer'


class BaseSerializer(object):

    """ Abstract class for serializers. """

    def __init__(
            self, scheme=None, options=None, format='django', **model_options):
        self.scheme = scheme
        self.format = format
        self.serializer_options = options or dict()
        self.model_options = self.init_options(**model_options)

    @staticmethod
    def init_options(fields=None, include=None, exclude=None, related=None):
        options = dict(
            fields=set(as_tuple(fields)),
            include=set(as_tuple(include)),
            exclude=set(as_tuple(exclude)),
            related=related or dict(),
        )
        return options

    def to_simple(self, value, **options):  # nolint
        " Simplify object. "

        # (string, unicode)
        if isinstance(value, basestring):
            return smart_unicode(value)

        # (int, long, float, real, complex, decimal)
        if isinstance(value, Number):
            return float(str(value)) if isinstance(value, Decimal) else value

        # (datetime, data, time)
        if isinstance(value, (datetime, date, time)):
            return self.to_simple_datetime(value)

        # (dict, ordereddict, mutable mapping)
        if isinstance(value, collections.MutableMapping):
            return dict(
                (k, self.to_simple(v, **options)) for k, v in value.items())

        # (tuple, list, set, iterators)
        if isinstance(value, collections.Iterable):
            return [self.to_simple(o, **options) for o in value]

        # (None, True, False)
        if value is None or value is True or value is False:
            return value

        if hasattr(value, 'to_simple') and not inspect.isclass(value):
            return self.to_simple(value.to_simple(self), **options)

        if isinstance(value, Model):
            return self.to_simple_model(value, **options)

        return str(value)

    @staticmethod
    def to_simple_datetime(value):
        result = value.isoformat()
        if isinstance(value, datetime):
            if value.microsecond:
                result = result[:23] + result[26:]
            if result.endswith('+00:00'):
                result = result[:-6] + 'Z'
        elif isinstance(value, time) and value.microsecond:
            result = result[:12]
        return result

    def to_simple_model(self, instance, **options): # noqa
        """ Convert model to simple python structure.
        """
        options = self.init_options(**options)
        fields, include, exclude, related = options['fields'], options['include'], options['exclude'], options['related'] # noqa

        result = dict(
            model=smart_unicode(instance._meta),
            pk=smart_unicode(
                instance._get_pk_val(), strings_only=True),
            fields=dict(),
        )

        default_fields = set([field.name for field in instance._meta.fields if field.serialize])
        serialized_fields = fields or (default_fields | include) - exclude

        for fname in serialized_fields:

            # Respect `to_simple__<fname>`
            to_simple = getattr(
                self.scheme, 'to_simple__{0}'.format(fname), None)

            if to_simple:
                result['fields'][fname] = to_simple(instance, serializer=self)
                continue

            related_options = related.get(fname, dict())
            if related_options:
                related_options = self.init_options(**related_options)

            if fname in default_fields and not related_options:
                field = instance._meta.get_field(fname)
                value = field.value_from_object(instance)

            else:
                value = getattr(instance, fname, None)
                if isinstance(value, Manager):
                    value = value.all()

            result['fields'][fname] = self.to_simple(
                value, **related_options)

        if self.format != 'django':
            fields = result['fields']
            fields['id'] = result['pk']
            result = fields

        return result

    def serialize(self, value):
        simple = self.to_simple(value, **self.model_options)
        if self.scheme:
            to_simple = getattr(self.scheme, 'to_simple', lambda s: s)
            simple = to_simple(value, simple, serializer=self)

        return simple


class JSONSerializer(BaseSerializer):

    def serialize(self, value):
        simple = super(JSONSerializer, self).serialize(value)
        return js.dumps(simple, **self.serializer_options)


class XMLSerializer(BaseSerializer):

    def serialize(self, value):
        simple = super(XMLSerializer, self).serialize(value)
        return ''.join(s for s in self._dumps(simple))

    def _dumps(self, value):  # nolint
        tag = it = None

        if isinstance(value, list):
            tag = 'items'
            it = iter(value)

        elif isinstance(value, dict) and 'model' in value:
            tag = value.get('model').split('.')[1]
            it = value.iteritems()

        elif isinstance(value, dict):
            it = value.iteritems()

        elif isinstance(value, tuple):
            tag = str(value[0])
            it = (i for i in value[1:])

        else:
            yield str(value)

        if tag:
            yield "<%s>" % tag

        if it:
            try:
                while True:
                    v = next(it)
                    yield ''.join(self._dumps(v))
            except StopIteration:
                yield ''

        if tag:
            yield "</%s>" % tag


# lint_ignore=W901,R0911,W0212,W0622



"""
    Immutable settings.
    Common for all projects.

"""
from os import path as op
import logging

ENVIRONMENT_NAME = 'core'

PROJECT_ROOT = op.abspath(op.dirname(op.dirname(__file__)))
PROJECT_NAME = "{0}.{1}".format(
    op.basename(op.dirname(PROJECT_ROOT)), op.basename(PROJECT_ROOT))

SECRET_KEY = "CHANGE_ME_{0}".format(PROJECT_NAME)

# Databases
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'django_master.sqlite',
        'USER': '',
        'PASSWORD': '',
        'TEST_CHARSET': 'utf8',
    }
}

# Caches
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'KEY_PREFIX': '_'.join((PROJECT_NAME, 'CORE'))
    }
}

# Base urls config
ROOT_URLCONF = 'main.urls'

# Media settigns
MEDIA_ROOT = op.join(PROJECT_ROOT, 'media')
STATIC_ROOT = op.join(PROJECT_ROOT, 'static')
MEDIA_URL = '/media/'
STATIC_URL = '/static/'

# Templates settings
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.core.context_processors.static',
    'django.core.context_processors.request',
    'django.contrib.auth.context_processors.auth',
    'django.contrib.messages.context_processors.messages',
)

# Applications
INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.admin',
    'django.contrib.staticfiles',
)

# Base apps settings
MESSAGE_STORAGE = 'django.contrib.messages.storage.cookie.CookieStorage'

# Middleware
MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

# Localization
USE_I18N = True
USE_L10N = True
USE_TZ = True
MIDDLEWARE_CLASSES += ('django.middleware.locale.LocaleMiddleware',)
TEMPLATE_CONTEXT_PROCESSORS += ('django.core.context_processors.i18n',)

# Debug
INTERNAL_IPS = ('127.0.0.1',)

# Logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
    datefmt='%d.%m %H:%M:%S',
)
logging.info("Core settings loaded.")



#!/usr/bin/env python
# coding: utf-8
from django.contrib.auth.models import User
from django.db import models


class Author(models.Model):
    name = models.CharField(max_length=100,
            help_text=u"Имя автора")
    user = models.ForeignKey(User)
    active = models.BooleanField(default=True)


class Publisher(models.Model):
    title = models.CharField(max_length=100)


class Book(models.Model):
    title = models.CharField(max_length=100)
    status = models.IntegerField(choices=(
        (1, 'new'),
        (2, 'published'),
        (3, 'archived'),
    ))
    author = models.ForeignKey(Author)
    price = models.PositiveIntegerField(default=0, blank=True)
    publisher = models.ForeignKey(Publisher, null=True, blank=True)
    books = models.ManyToManyField('self', blank=True)


class Article(models.Model):
    title = models.CharField(max_length=100)
    book = models.ForeignKey(Book)



""" Base request resource. """

from django.conf.urls import url
from django.core.exceptions import ValidationError
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View
from logging import getLogger

from .mixin import auth, emitter, handler, parser, throttle
from .settings import ADREST_ALLOW_OPTIONS, ADREST_DEBUG
from .signals import api_request_started, api_request_finished
from .utils import status
from .utils.exceptions import HttpError, FormError
from .utils.mail import adrest_errors_mail
from .utils.response import SerializedHttpResponse
from .utils.tools import as_tuple, gen_url_name, gen_url_regex, fix_request


logger = getLogger('django.request')


__all__ = 'ResourceView',


class ResourceMetaClass(
        handler.HandlerMeta, throttle.ThrottleMeta, emitter.EmitterMeta,
        parser.ParserMeta, auth.AuthMeta):

    """ MetaClass for ResourceView. Create meta options. """

    def __new__(mcs, name, bases, params):

        # Run other meta classes
        cls = super(ResourceMetaClass, mcs).__new__(mcs, name, bases, params)

        meta = params.get('Meta')

        cls._meta.abstract = meta and getattr(meta, 'abstract', False)
        if cls._meta.abstract:
            return cls

        # Meta name (maybe precalculate in handler)
        cls._meta.name = cls._meta.name or ''.join(
            bit for bit in name.split('Resource') if bit).lower()

        # Prepare urls
        cls._meta.url_params = list(as_tuple(cls._meta.url_params))
        cls._meta.url_name = cls._meta.url_name or '-'.join(gen_url_name(cls))
        cls._meta.url_regex = cls._meta.url_regex or '/'.join(
            gen_url_regex(cls))

        return cls


class ResourceView(
        handler.HandlerMixin, throttle.ThrottleMixin, emitter.EmitterMixin,
        parser.ParserMixin, auth.AuthMixin, View):

    """ REST Resource. """

    # Create meta options
    __metaclass__ = ResourceMetaClass

    # Link to api if connected
    api = None

    # Instance's identifier
    identifier = None

    class Meta:

        # This abstract class
        abstract = True

        # Name (By default this set from model or class name)
        name = None

        # Save access log if ADRest logging is enabled
        log = True

        # Some custom URI params here
        url_params = None
        url_regex = None
        url_name = None

        # Custom prefix for url name and regex
        prefix = ''

        # If children object in hierarchy has FK=Null to parent,
        # allow to get this object (default: True)
        allow_public_access = False

    @csrf_exempt
    def dispatch(self, request, **resources):
        """ Try to dispatch the request.

        :return object: result

        """

        # Fix PUT and PATH methods in Django request
        request = fix_request(request)

        # Set self identifier
        self.identifier = request.META.get('REMOTE_ADDR', 'anonymous')

        # Send ADREST started signal
        api_request_started.send(self, request=request)

        # Send current api started signal
        if self.api:
            self.api.request_started.send(self, request=request)

        try:

            # Check request method
            self.check_method_allowed(request)

            # Authentificate
            self.authenticate(request)

            # Throttle check
            self.throttle_check()

            if request.method != 'OPTIONS' or not ADREST_ALLOW_OPTIONS:

                # Parse content
                request.data = self.parse(request)

                # Get required resources
                resources = self.get_resources(
                    request, **resources)

                # Check owners
                self.check_owners(request, **resources)

                # Check rights for resources with this method
                self.check_rights(resources, request=request)

            response = self.handle_request(request, **resources)

            # Serialize response
            response = self.emit(response, request=request)

        except Exception as e:
            response = self.handle_exception(e, request=request)

        response["Allow"] = ', '.join(self._meta.allowed_methods)
        response["Vary"] = 'Authenticate, Accept'

        # Send errors on mail
        adrest_errors_mail(response, request)

        # Send finished signal
        api_request_finished.send(
            self, request=request, response=response, **resources)

        # Send finished signal in API context
        if self.api:
            self.api.request_finished.send(
                self, request=request, response=response, **resources)

        return response

    def check_owners(self, request, **resources):
        """ Check parents of current resource.

        Recursive scanning of the fact that the child has FK
        to the parent and in resources we have right objects.

        We check that in request like /author/1/book/2/page/3

        Page object with pk=3 has ForeignKey field linked to Book object
        with pk=2 and Book with pk=2 has ForeignKey field linked to Author
        object with pk=1.

        :return bool: If success else raise Exception

        """

        if self._meta.allow_public_access or not self._meta.parent:
            return True

        self.parent.check_owners(request, **resources)

        objects = resources.get(self._meta.name)
        if self._meta.model and self._meta.parent._meta.model and objects:
            pr = resources.get(self._meta.parent._meta.name)
            check = all(
                pr.pk == getattr(
                    o, "%s_id" % self._meta.parent._meta.name, None)
                for o in as_tuple(objects))

            if not pr or not check:
                # 403 Error if there is error in parent-children relationship
                raise HttpError(
                    "Access forbidden.", status=status.HTTP_403_FORBIDDEN)

        return True

    def handle_exception(self, e, request=None):
        """ Handle code exception.

        :return response: Http response

        """
        if isinstance(e, HttpError):
            response = SerializedHttpResponse(e.content, status=e.status)
            return self.emit(
                response, request=request, emitter=e.emitter)

        if isinstance(e, (AssertionError, ValidationError)):

            content = unicode(e)

            if isinstance(e, FormError):
                content = e.form.errors

            response = SerializedHttpResponse(
                content, status=status.HTTP_400_BAD_REQUEST)

            return self.emit(response, request=request)

        if ADREST_DEBUG:
            raise

        logger.exception('\nADREST API Error: %s', request.path)

        return HttpResponse(str(e), status=500)

    @classmethod
    def as_url(cls, api=None, name_prefix='', url_prefix=''):
        """ Generate url for resource.

        :return RegexURLPattern: Django URL

        """
        url_prefix = url_prefix and "%s/" % url_prefix
        name_prefix = name_prefix and "%s-" % name_prefix

        url_regex = '^%s%s/?$' % (
            url_prefix, cls._meta.url_regex.lstrip('^').rstrip('/$'))
        url_regex = url_regex.replace('//', '/')
        url_name = '%s%s' % (name_prefix, cls._meta.url_name)

        return url(url_regex, cls.as_view(api=api), name=url_name)


# pylama:ignore=E1120,W0703



""" ADRest authentication support.
"""
from ..settings import ADREST_ALLOW_OPTIONS
from ..utils import status
from ..utils.meta import MixinBaseMeta
from ..utils.auth import AnonimousAuthenticator, AbstractAuthenticator
from ..utils.exceptions import HttpError
from ..utils.tools import as_tuple


__all__ = 'AuthMixin',


class AuthMeta(MixinBaseMeta):

    """ Convert cls.meta.authenticators to tuple and check them. """

    def __new__(mcs, name, bases, params):
        cls = super(AuthMeta, mcs).__new__(mcs, name, bases, params)

        cls._meta.authenticators = as_tuple(cls._meta.authenticators)

        if not cls._meta.authenticators:
            raise AssertionError("Should be defined at least one authenticator.")

        for a in cls._meta.authenticators:
            if not issubclass(a, AbstractAuthenticator):
                raise AssertionError("Meta.authenticators should be subclasses of "
                                     "`adrest.utils.auth.AbstractAuthenticator`")

        return cls


class AuthMixin(object):

    """ Adds pluggable authentication behaviour. """

    __metaclass__ = AuthMeta

    class Meta:
        authenticators = AnonimousAuthenticator

    def __init__(self, *args, **kwargs):
        self.auth = None

    def authenticate(self, request):
        """ Attempt to authenticate the request.

        :param request: django.http.Request instance

        :return bool: True if success else raises HTTP_401

        """
        authenticators = self._meta.authenticators

        if request.method == 'OPTIONS' and ADREST_ALLOW_OPTIONS:
            self.auth = AnonimousAuthenticator(self)
            return True

        error_message = "Authorization required."
        for authenticator in authenticators:
            auth = authenticator(self)
            try:
                if not auth.authenticate(request):
                    raise AssertionError(error_message)

                self.auth = auth
                auth.configure(request)
                return True
            except AssertionError, e:
                error_message = str(e)

        raise HttpError(error_message, status=status.HTTP_401_UNAUTHORIZED)

    def check_rights(self, resources, request=None):
        """ Check rights for resources.

        :return bool: True if operation is success else HTTP_403_FORBIDDEN

        """
        if not self.auth:
            return True

        try:
            if not self.auth.test_rights(resources, request=request):
                raise AssertionError()

        except AssertionError, e:
            raise HttpError("Access forbiden. {0}".format(e), status=status.HTTP_403_FORBIDDEN)



from django.contrib import admin
from django.db.models.loading import get_models, get_app


# Register main models in admin
app = get_app('main')
for model in get_models(app):
    admin.site.register(model, admin.ModelAdmin)



""" Pagination support. """

from urllib import urlencode

from django.core.paginator import InvalidPage, Paginator as DjangoPaginator

from .exceptions import HttpError
from .status import HTTP_400_BAD_REQUEST


class Paginator(object):

    """ Paginate collections. """

    def __init__(self, request, resource, response):
        self.query_dict = dict(request.GET.items())
        self.path = request.path

        try:
            per_page = resource._meta.dyn_prefix + 'max'
            self.paginator = DjangoPaginator(
                response,
                self.query_dict.get(per_page) or resource._meta.limit_per_page)

            if not self.paginator.per_page:
                self.paginator = None

        except (ValueError, AssertionError):
            self.paginator = None

        self._page = None

    def to_simple(self, serializer=None):
        """ Prepare to serialization.

        :return dict: paginator params

        """
        return dict(
            count=self.paginator.count,
            page=self.page_number,
            num_pages=self.paginator.num_pages,
            next=self.next_page,
            prev=self.previous_page,
            resources=self.resources,
        )

    @property
    def page(self):
        """ Get current page.

        :return int: page number

        """
        if not self._page:
            try:
                self._page = self.paginator.page(
                    self.query_dict.get('page', 1))
            except InvalidPage:
                raise HttpError("Invalid page", status=HTTP_400_BAD_REQUEST)
        return self._page

    @property
    def page_number(self):
        """Get page number

        :return: int

        """
        return self.page.number if self.page else 1

    @property
    def count(self):
        """ Get resources count.

        :return int: resources amount

        """
        return self.paginator.count

    @property
    def resources(self):
        """ Return list of current page resources.

        :return list:

        """
        return self.page.object_list

    @property
    def next_page(self):
        """ Return URL for next page.

        :return str:

        """
        if self.page.has_next():
            self.query_dict['page'] = self.page.next_page_number()
            return "%s?%s" % (self.path, urlencode(self.query_dict))
        return ""

    @property
    def previous_page(self):
        """ Return URL for previous page.

        :return str:

        """
        if self.page.has_previous():
            previous = self.page.previous_page_number()
            if previous == 1:
                if 'page' in self.query_dict:
                    del self.query_dict['page']
            else:
                self.query_dict['page'] = previous
            return "%s?%s" % (self.path, urlencode(self.query_dict))
        return ""



""" Implement REST functionality. """
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
from django.http import HttpResponse
from logging import getLogger

from ..forms import PartitialForm
from ..settings import ADREST_ALLOW_OPTIONS
from ..utils import status, UpdatedList
from ..utils.exceptions import HttpError, FormError
from ..utils.tools import as_tuple
from .dynamic import DynamicMixin, DynamicMixinMeta


__all__ = 'HandlerMixin',

logger = getLogger('django.request')


__all__ = 'HandlerMixin',


class Meta:

    """ Handler options. Setup parameters for REST implementation.

    ::

        class SomeResource(HandlerMixin, View):

            class Meta:
                allowed_methods = 'get', 'post'
                model = 'app.model'

    """

    #: List of allowed methods (or simple one)
    allowed_methods = 'GET',

    #: Map HTTP methods to handler methods
    callmap = dict(
        (m.upper(), m) for m in (
            'get', 'post', 'put', 'delete', 'patch', 'options', 'head')
    )

    #: Set form for resource by manual
    form = None

    #: Specify field's names for automatic a model form
    form_fields = None

    #: Exclude field's names for automatic a model form
    form_exclude = None


class HandlerMeta(DynamicMixinMeta):

    """ Prepare handler class. """

    def __new__(mcs, name, bases, params):

        cls = super(HandlerMeta, mcs).__new__(mcs, name, bases, params)

        # Prepare allowed methods
        cls._meta.allowed_methods = mcs.__prepare_methods(
            cls._meta.allowed_methods)

        if not cls._meta.model:
            return cls

        cls._meta.name = cls._meta.name or cls._meta.model._meta.module_name

        # Create form if not exist
        if not cls._meta.form:

            class DynForm(PartitialForm):

                class Meta:
                    model = cls._meta.model
                    fields = cls._meta.form_fields
                    exclude = cls._meta.form_exclude

            cls._meta.form = DynForm

        return cls

    @staticmethod
    def __prepare_methods(methods):

        methods = tuple([str(m).upper() for m in as_tuple(methods)])

        if not 'OPTIONS' in methods and ADREST_ALLOW_OPTIONS:
            methods += 'OPTIONS',

        if not 'HEAD' in methods and 'GET' in methods:
            methods += 'HEAD',

        return methods


class HandlerMixin(DynamicMixin):

    """ Implement REST API.


    .. autoclass:: adrest.mixin.handler.Meta
       :members:

    Example: ::

        class SomeResource(HandlerMixin, View):

            class Meta:
                allowed_methods = 'get', 'post'
                model = 'app.model'

            def dispatch(self, request, **resources):

                self.check_method_allowed(request)

                resources = self.get_resources(request, **resources)

                return self.handle_request(request, **resources)

    """

    __metaclass__ = HandlerMeta

    Meta = Meta

    def handle_request(self, request, **resources):
        """ Get a method for request and execute.

        :return object: method result

        """
        if not request.method in self._meta.callmap.keys():
            raise HttpError(
                'Unknown or unsupported method \'%s\'' % request.method,
                status=status.HTTP_501_NOT_IMPLEMENTED)

        # Get the appropriate create/read/update/delete function
        view = getattr(self, self._meta.callmap[request.method])

        # Get function data
        return view(request, **resources)

    @staticmethod
    def head(*args, **kwargs):
        """ Just return empty response.

        :return django.http.Response: empty response.

        """

        return HttpResponse()

    def get(self, request, **resources):
        """ Default GET method. Return instance (collection) by model.

        :return object: instance or collection from self model

        """

        instance = resources.get(self._meta.name)
        if not instance is None:
            return instance

        return self.paginate(
            request, self.get_collection(request, **resources))

    def post(self, request, **resources):
        """ Default POST method. Uses the handler's form.

        :return object: saved instance or raise form's error

        """
        if not self._meta.form:
            return None

        form = self._meta.form(request.data, **resources)
        if form.is_valid():
            return form.save()

        raise FormError(form)

    def put(self, request, **resources):
        """ Default PUT method. Uses self form. Allow bulk update.

        :return object: changed instance or raise form's error

        """
        if not self._meta.form:
            return None

        if not self._meta.name in resources or not resources[self._meta.name]:
            raise HttpError(
                "Resource not found.", status=status.HTTP_404_NOT_FOUND)
        resource = resources.pop(self._meta.name)

        updated = UpdatedList()
        for o in as_tuple(resource):
            form = self._meta.form(data=request.data, instance=o, **resources)

            if not form.is_valid():
                raise FormError(form)

            updated.append(form.save())

        return updated if len(updated) > 1 else updated[-1]

    def delete(self, request, **resources):
        """ Default DELETE method. Allow bulk delete.

        :return django.http.response: empty response

        """

        resource = resources.get(self._meta.name)
        if not resource:
            raise HttpError("Bad request", status=status.HTTP_404_NOT_FOUND)

        for o in as_tuple(resource):
            o.delete()

        return HttpResponse("")

    def patch(self, request, **resources):
        """ Default PATCH method.

        :return object: changed instance or raise form's error

        """
        return self.put(request, **resources)

    @staticmethod
    def options(request, **resources):
        """ Default OPTIONS method.

        :return django.http.response: 'OK' response

        """

        return HttpResponse("OK")

    @classmethod
    def check_method_allowed(cls, request):
        """ Ensure the request HTTP method is permitted for this resource.

        Raising a ResourceException if it is not.

        """
        if not request.method in cls._meta.allowed_methods:
            raise HttpError(
                'Method \'%s\' not allowed on this resource.' % request.method,
                status=status.HTTP_405_METHOD_NOT_ALLOWED)

    def get_resources(self, request, **resources):
        """ Parse resource objects from URL and request.

        :return dict: Resources.

        """

        if self.parent:
            resources = self.parent.get_resources(request, **resources)

        pks = (
            resources.get(self._meta.name) or
            request.REQUEST.getlist(self._meta.name) or
            getattr(request, 'data', None) and request.data.get(
                self._meta.name))

        if not pks or self._meta.queryset is None:
            return resources

        pks = as_tuple(pks)

        try:
            if len(pks) == 1:
                resources[self._meta.name] = self._meta.queryset.get(pk=pks[0])

            else:
                resources[self._meta.name] = self._meta.queryset.filter(
                    pk__in=pks)

        except (ObjectDoesNotExist, ValueError, AssertionError):
            raise HttpError("Resource not found.",
                            status=status.HTTP_404_NOT_FOUND)

        except MultipleObjectsReturned:
            raise HttpError("Resources conflict.",
                            status=status.HTTP_409_CONFLICT)

        return resources


# pymode:lint_ignore=E1102,W0212,R0924



from adrest.api import Api
from adrest.utils.auth import AnonimousAuthenticator
from adrest.utils.emitter import XMLEmitter, JSONTemplateEmitter
from adrest.views import ResourceView
from adrest.resources.rpc import RPCResource, JSONEmitter, JSONPEmitter


API = Api('1.0.0', api_rpc=True, emitters=XMLEmitter)


class TestAuth(AnonimousAuthenticator):

    def authenticate(self, request):
        return request.META.get('HTTP_AUTHORIZATION')

    def configure(self, request):
        self.resource.identifier = request.META.get('HTTP_AUTHORIZATION')


class TestResource(ResourceView):

    class Meta:
        allowed_methods = 'GET', 'POST', 'PUT'
        model = 'rpc.test'

    def get(self, request, **resources):
        assert not 'error' in request.GET, "Custom error"
        return True


class RootResource(ResourceView):

    class Meta:
        allowed_methods = 'GET', 'POST', 'PUT'
        model = 'rpc.root'


@API.register
class ChildResource(ResourceView):

    class Meta:
        allowed_methods = 'GET', 'POST', 'PUT'
        parent = RootResource
        model = 'rpc.child'


class CustomResource(ResourceView):

    class Meta:
        model = 'rpc.custom'


API.register(ChildResource)
API.register(CustomResource, emitters=JSONTemplateEmitter)
API.register(RootResource, authenticators=TestAuth)
API.register(RPCResource, url_regex=r'^rpc2$', url_name='rpc2',
             scheme='tests.rpc.dummy', emitters=(JSONEmitter, JSONPEmitter))
API.register(TestResource)

# lint_ignore=C



from ..api import api as API


class AutoJSONRPCTest(API.testCase):

    def test_base(self):
        self.assertTrue('autojsonrpc' in self.api.resources)

        uri = self.reverse('autojsonrpc')
        self.assertEqual(uri, '/pirates/rpc')

        response = self.get_resource('autojsonrpc')
        self.assertContains(response, 'Invalid RPC Call.')

# lint_ignore=C0110



from mixer.backend.django import mixer

from ..api import api as API
from adrest.tests import AdrestTestCase, reverse


class CoreAdrestTests(AdrestTestCase):

    api = API

    def test_json(self):
        pirate = mixer.blend('core.pirate', character='good')
        url = reverse('pirate', API)
        self.assertEqual(url, '/pirates/pirate/')

        response = self.put_resource(
            'pirate', pirate=pirate, json=True, data=dict(name='John'))
        self.assertContains(response, '"name": "John"')


# lint_ignore=F0401,C



""" Collect ADRest mixins.
"""

from .auth import AuthMixin
from .dynamic import DynamicMixin
from .emitter import EmitterMixin
from .handler import HandlerMixin
from .parser import ParserMixin
from .throttle import ThrottleMixin

# lint_ignore=W0611



""" ADRest parse data. """
from ..utils.meta import MixinBaseMeta
from ..utils.parser import FormParser, XMLParser, JSONParser, AbstractParser
from ..utils.tools import as_tuple

__all__ = 'ParserMixin',


class ParserMeta(MixinBaseMeta):

    """ Prepare resource's parsers. """

    def __new__(mcs, name, bases, params):
        cls = super(ParserMeta, mcs).__new__(mcs, name, bases, params)

        cls._meta.parsers = as_tuple(cls._meta.parsers)
        if not cls._meta.parsers:
            raise AssertionError("Should be defined at least one parser.")

        cls._meta.default_parser = cls._meta.parsers[0]
        cls._meta.parsers_dict = dict()

        for p in cls._meta.parsers:
            if not issubclass(p, AbstractParser):
                raise AssertionError(
                    "Parser must be subclass of AbstractParser.")

            cls._meta.parsers_dict[p.media_type] = p

        return cls


class ParserMixin(object):

    """ Parse user data. """

    __metaclass__ = ParserMeta

    class Meta:
        parsers = FormParser, XMLParser, JSONParser

    def parse(self, request):
        """ Parse request content.

        :return dict: parsed data.

        """
        if request.method in ('POST', 'PUT', 'PATCH'):
            content_type = self.determine_content(request)
            if content_type:
                split = content_type.split(';', 1)
                if len(split) > 1:
                    content_type = split[0]
                content_type = content_type.strip()

            parser = self._meta.parsers_dict.get(
                content_type, self._meta.default_parser)
            data = parser(self).parse(request)
            return dict() if isinstance(data, basestring) else data
        return dict()

    @staticmethod
    def determine_content(request):
        """ Determine request content.

        :return str: request content type

        """

        if not request.META.get('CONTENT_LENGTH', None) \
           and not request.META.get('TRANSFER_ENCODING', None):
            return None

        return request.META.get('CONTENT_TYPE', None)



from adrest.tests.utils import AdrestTestCase
from mixer.backend.django import mixer
import json as js

from .api import API
from .models import Root, Child


class RPCTestCase(AdrestTestCase):
    api = API

    def setUp(self):
        self.root1 = Root.objects.create(name='test_root1')
        for i in xrange(10):
            Child.objects.create(root=self.root1, name='test_child1')

        self.root2 = Root.objects.create(name='test_root2')
        for i in xrange(10):
            Child.objects.create(
                root=self.root2, name='test_child2', odd=i % 2)

    def test_base_rpc(self):

        response = self.get_resource('rpc2', method='options')
        self.assertEqual(response.content, 'OK')

        # POST args
        response = self.rpc(
            'rpc2',
            rpc=dict(
                jsonrpc='2.0',
                method='method1',
                params=['test'],
            ))
        self.assertEqual(response.content, '"Hello test"')

        # POST kwargs
        response = self.rpc(
            'rpc2',
            rpc=dict(
                jsonrpc='2.0',
                method='method2',
                params=dict(
                    start=200,
                    end=300
                )
            ))
        response = js.loads(response.content)
        self.assertTrue(200 <= response <= 300)

        response = self.rpc(
            'rpc2',
            rpc=dict(
                jsonrpc='2.0',
                method='wrongmethodname',
            ))
        self.assertContains(response, "Unknown method")

        # Handle Errors
        response = self.rpc(
            'rpc2',
            rpc=dict(
                jsonrpc='2.0',
                method='error_method',
            ))
        response = js.loads(response.content)
        self.assertEqual(response['error']['message'], 'Error here')

        # GET JSONRPC
        response = self.rpc(
            'rpc2',
            callback='answer',
            rpc=dict(
                jsonrpc='2.0',
                method='method1',
                params=['test'],
            ))
        self.assertEqual(response.content, 'answer("Hello test")')

    def test_base(self):
        uri = self.reverse('test')
        self.assertEqual(uri, '/rpc/1.0.0/test/')

        response = self.get_resource('test')
        self.assertContains(response, 'true')

    def test_autojsonrpc(self):
        uri = self.reverse('autojsonrpc')
        self.assertEqual(uri, '/rpc/1.0.0/rpc')

        response = self.get_resource('autojsonrpc')
        self.assertContains(response, 'Invalid RPC Call.')

        response = self.rpc('autojsonrpc', callback='answer', rpc=dict(method="iamwrongmethod"))
        self.assertContains(response, 'Wrong method')

        response = self.rpc('autojsonrpc', callback='answer', rpc=dict(method='bla.bla'))
        self.assertContains(response, 'Unknown method')

        response = self.rpc('autojsonrpc', rpc=dict(method='test.bla'))
        self.assertContains(response, 'not allowed')

        response = self.rpc('autojsonrpc', rpc=dict(method='test.get'))
        self.assertContains(response, 'true')

        response = self.rpc('autojsonrpc', rpc=dict(method='root.get'))
        self.assertContains(response, 'Authorization required')

        response = self.rpc(
            'autojsonrpc',
            rpc=dict(
                headers=dict(Authorization=111),
                method='root.get'))
        self.assertContains(response, 'test_root')

        response = self.rpc(
            'autojsonrpc',
            rpc=dict(
                params=dict(root=self.root1.pk),
                method='root-child.get'))
        self.assertContains(response, '"count": 10')
        self.assertContains(response, 'test_child1')
        self.assertNotContains(response, 'test_child2')

        response = self.rpc(
            'autojsonrpc',
            rpc=dict(
                params=dict(root=self.root2.pk),
                data=dict(odd=1),
                method='root-child.get'))
        self.assertContains(response, '"count": 5')

        response = self.rpc(
            'autojsonrpc',
            key=111,
            rpc=dict(
                data=dict(name='root3'),
                method='root.post'))
        self.assertContains(response, 'root3')

        response = self.rpc(
            'autojsonrpc',
            key=111,
            rpc=dict(
                data=dict(name='child3'),
                params=dict(root=self.root1.pk),
                method='root-child.post'))
        self.assertContains(response, 'child3')

        child = Child.objects.get(name='child3')
        self.assertEqual(child.root, self.root1)

        response = self.rpc(
            'autojsonrpc',
            key=111,
            rpc=dict(
                data=dict(name='child4', root=self.root2.pk),
                params=dict(root=self.root1.pk),
                method='root-child.post'))
        self.assertContains(response, 'child4')

        child = Child.objects.get(name='child4')
        self.assertEqual(child.root, self.root1)

        response = self.rpc(
            'autojsonrpc',
            key=111,
            rpc=dict(
                data=dict(name='New name'),
                params=dict(root=self.root1.pk,
                            child=self.root1.child_set.all()[0].pk),
                method='root-child.put'))
        self.assertContains(response, 'New name')
        child = self.root1.child_set.all()[0]
        self.assertEqual(child.name, 'New name')

        response = self.rpc(
            'autojsonrpc',
            callback='test1234',
            rpc=dict(method='test.get'))
        self.assertContains(response, 'test1234')

        response = self.rpc(
            'autojsonrpc',
            callback='test',
            rpc=dict(
                method='test.get',
                callback='test1234'))
        self.assertContains(response, 'test1234')

    def test_custom(self):
        mixer.blend('rpc.custom')
        response = self.rpc(
            'autojsonrpc',
            rpc=dict(method='custom.get'))
        self.assertContains(response, 'Custom template')

    def test_request(self):
        response = self.rpc(
            'rpc2',
            rpc=dict(
                jsonrpc='2.0',
                method='method3',
                params=['test'],
            ))
        self.assertEqual(response.content, '"POSTtest"')

    def test_private(self):
        response = self.rpc(
            'rpc2',
            rpc=dict(
                jsonrpc='2.0',
                method='__private_method',
            ))
        self.assertContains(response, 'error')

# lint_ignore=C,F0401



""" .

Configuration
=============

You should add ``adrest`` to your ``INSTALLED_APPS`` in Django settings.

Also you can redefine default **ADRest** settings writen bellow.


"""
try:
    from django.core.exceptions import ImproperlyConfigured
    from django.conf import settings

    getattr(settings, 'DEBUG')

except (ImportError, ImproperlyConfigured):

    settings.configure()

from .utils.tools import as_tuple


#: Enable ADRest API logs. Information about requests and responses will be
#: saved in database.
ADREST_ACCESS_LOG = getattr(settings, 'ADREST_ACCESS_LOG', False)

#: Create `adrest.models.AccessKey` models for authorisation by keys
ADREST_ACCESSKEY = getattr(
    settings, 'ADREST_ACCESSKEY',
    'django.contrib.auth' in getattr(settings, 'INSTALLED_APPS', tuple()))

#: Create AccessKey for Users automaticly
ADREST_AUTO_CREATE_ACCESSKEY = getattr(
    settings, 'ADREST_AUTO_CREATE_ACCESSKEY', False)

#: Set default number resources per page for pagination
#: ADREST_LIMIT_PER_PAGE = 0 -- Disabled pagination by default
ADREST_LIMIT_PER_PAGE = int(getattr(settings, 'ADREST_LIMIT_PER_PAGE', 50))

#: Dont parse a exceptions. Show standart Django 500 page.
ADREST_DEBUG = getattr(settings, 'ADREST_DEBUG', False)

#: List of errors for ADRest's errors mails.
#: Set ADREST_MAIL_ERRORS = None for disable this functionality
ADREST_MAIL_ERRORS = as_tuple(getattr(settings, 'ADREST_MAIL_ERRORS', 500))

#: Set maximum requests per timeframe
ADREST_THROTTLE_AT = getattr(settings, 'ADREST_THROTTLE_AT', 120)

#: Set timeframe length
ADREST_THROTTLE_TIMEFRAME = getattr(settings, 'ADREST_THROTTLE_TIMEFRAME', 60)

#: We do not restrict access for OPTIONS request.
ADREST_ALLOW_OPTIONS = getattr(settings, 'ADREST_ALLOW_OPTIONS', False)

#: Template path for ADRest map
ADREST_MAP_TEMPLATE = getattr(settings, 'ADREST_MAP_TEMPLATE', 'api/map.html')



import json as js
from collections import defaultdict
from django.conf import settings
from django.core.urlresolvers import reverse as django_reverse
from django.db.models import Model
from django.test import client
from django.utils.encoding import smart_str
from django.utils.functional import curry
from django.utils.http import urlencode
from urlparse import urlparse


try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


def generic_method(rf, path, data=None, content_type=client.MULTIPART_CONTENT, follow=False,
                   method='PUT', **extra):
    """ Fix django.

    :return request: request

    """
    if content_type is client.MULTIPART_CONTENT:
        data = rf._encode_data(data, content_type)
    else:
        data = smart_str(data, encoding=settings.DEFAULT_CHARSET)

    parsed = urlparse(path)
    r = {
        'CONTENT_LENGTH': len(data),
        'CONTENT_TYPE': content_type,
        'PATH_INFO': rf._get_path(parsed),
        'QUERY_STRING': parsed[4],
        'REQUEST_METHOD': method,
        'wsgi.input': FakePayload(data),
    }
    r.update(extra)
    return rf.request(**r)


class AdrestRequestFactory(client.RequestFactory):

    """ Path methods. """

    put = curry(generic_method, method="PUT")
    patch = curry(generic_method, method="PATCH")


class AdrestClient(client.Client):

    """ Patch client. """

    api = None

    def put(self, path, data=None, content_type=client.MULTIPART_CONTENT,
            follow=False, method='PUT', **extra):
        """ Implement PUT.

        :return response: A result.

        """
        data = data or dict()
        response = generic_method(
            self, path, data=data, content_type=content_type, follow=follow,
            method=method, **extra)
        if follow:
            response = self._handle_redirects(response, **extra)
        return response

    patch = curry(put, method='PATCH')

    def delete(self, path, data=None, **extra):
        """ Implement DELETE.

        :return response: A result.

        """
        data = data or dict()

        parsed = urlparse(path)
        r = {
            'PATH_INFO':       self._get_path(parsed),
            'QUERY_STRING':    urlencode(data, doseq=True) or parsed[4],
            'REQUEST_METHOD': 'DELETE',
        }
        r.update(extra)
        return self.request(**r)

    def request_resource(self, resource, method='get', data=None, headers=None, json=False,
                         api=None, key=None, **kwargs):
        """ Simply request resource method.

        :param resource: Resource Class or String name.
        :param data: Request data
        :param json: Make JSON request
        :param headers: Request headers
        :param client: Test client
        :param api: Resource API
        :param key: HTTP_AUTHORIZATION token

        :return object: result

        """
        method = getattr(self, method)
        headers = headers or dict()
        api = api or self.api

        if isinstance(key, Model):
            key = key.key

        headers['HTTP_AUTHORIZATION'] = key or headers.get('HTTP_AUTHORIZATION')

        data = data or dict()

        # Support JSON request
        if json:
            headers['content_type'] = 'application/json'
            data = js.dumps(data)

        url = self.reverse(resource, api=api, **kwargs)
        return jsonify(method(url, data=data, **headers))

    get_resource = request_resource
    put_resource = curry(request_resource, method='put')
    post_resource = curry(request_resource, method='post')
    patch_resource = curry(request_resource, method='patch')
    delete_resource = curry(request_resource, method='delete')

    def reverse(self, resource, api=None, **kwargs):
        api = api or self.api
        return reverse(resource, api=api, **kwargs)


class FakePayload(object):

    """ Fake payload.

    A wrapper around StringIO that restricts what can be read since data from
    the network can't be seeked and cannot be read outside of its content
    length. This makes sure that views can't do anything under the test client
    that wouldn't work in Real Life.

    """

    def __init__(self, content):
        self.__content = StringIO(content)
        self.__len = len(content)

    def read(self, num_bytes=None):
        """ READ.

        :return str: content

        """
        if num_bytes is None:
            num_bytes = self.__len or 0
        assert self.__len >= num_bytes, "Cannot read more than the available bytes from the HTTP incoming data."  # noqa
        content = self.__content.read(num_bytes)
        self.__len -= num_bytes
        return content


def reverse(resource, api=None, namespace=None, **resources):
    """ Reverse resource by ResourceClass or name string.

    :param resource: Resource Class or String name.
    :param api: API intance (if resource is string)
    :param namespace: Set namespace prefix
    :param **resources: Uri params

    :return str: URI string

    """
    if isinstance(resource, basestring):
        url_name = resource
        if not api:
            raise AssertionError("You sould send api parameter")

        if not api.resources.get(url_name):
            raise AssertionError("Invalid resource name: %s" % url_name)

    else:
        url_name = resource._meta.url_name
        api = resource.api

    params = dict()
    query = defaultdict(list)

    for name, resource in resources.items():

        if isinstance(resource, Model):
            resource = resource.pk

        if name in params:
            query[name].append(params[name])
            query[name].append(resource)
            del params[name]
            continue

        params[name] = resource

    name_ver = '' if not str(api) else '%s-' % str(api)
    ns_prefix = '' if not namespace else '%s:' % namespace
    uri = django_reverse(
        '%s%s-%s%s' % (ns_prefix, api.prefix, name_ver, url_name), kwargs=params)

    if query:
        uri += '?'
        for name, values in query:
            uri += '&'.join('%s=%s' % (name, value) for value in values)

    return uri


def jsonify(response):
    """ Check for request content is JSON. """
    if response.get('Content-type') == 'application/json':
        try:
            response.json = js.loads(response.content)
        except ValueError:
            return response
    return response

# pylama:ignore=D100,W0212,D102



""" RPC support. """
import json as js
from django.http import QueryDict, HttpResponse
from django.utils import importlib

from ..utils.emitter import JSONPEmitter, JSONEmitter
from ..utils.parser import JSONParser, FormParser
from ..utils.response import SerializedHttpResponse
from ..utils.tools import as_tuple
from ..views import ResourceView, ResourceMetaClass


__all__ = 'get_request', 'RPCResource', 'AutoJSONRPC'


def get_request(func):
    """ Mark function as needed in request.

    :return function: marked function.

    """
    func.request = True
    return func


class RPCMeta(ResourceMetaClass):

    """ Setup RPC methods by Scheme. """

    def __new__(mcs, name, bases, params):
        cls = super(RPCMeta, mcs).__new__(mcs, name, bases, params)
        cls.configure_rpc()
        return cls


class RPCResource(ResourceView):

    """ JSON RPC support.

    Implementation of remote procedure call encoded in JSON.
    Allows for notifications (info sent to the server that does not require
    a response) and for multiple calls to be sent to the server which may
    be answered out of order.

    """

    class Meta:
        allowed_methods = 'get', 'post'
        emitters = JSONEmitter, JSONPEmitter
        parsers = JSONParser, FormParser
        scheme = None
        url_regex = r'^rpc$'

    methods = dict()
    scheme_name = ''

    __metaclass__ = RPCMeta

    def __init__(self, scheme=None, **kwargs):
        if scheme:
            self.configure_rpc(scheme)
        super(RPCResource, self).__init__(**kwargs)

    @classmethod
    def configure_rpc(cls, scheme=None):
        """ Get methods from scheme. """
        scheme = scheme or cls._meta.scheme

        if not scheme:
            return

        if isinstance(scheme, basestring):
            scheme = importlib.import_module(scheme)

        cls.scheme_name = scheme.__name__

        methods = getattr(scheme, '__all__', None) \
            or [m for m in dir(scheme) if not m.startswith('_')]

        for mname in methods:
            method = getattr(scheme, mname)
            if hasattr(method, '__call__'):
                cls.methods["{0}.{1}".format(
                    cls.scheme_name, method.__name__)] = method

    def handle_request(self, request, **resources):
        """ Call RPC method.

        :return object: call's result

        """

        if request.method == 'OPTIONS':
            return super(RPCResource, self).handle_request(
                request, **resources)

        payload = request.data

        try:

            if request.method == 'GET':
                payload = request.GET.get('payload')
                try:
                    payload = js.loads(payload)
                except TypeError:
                    raise AssertionError("Invalid RPC Call.")

            if 'method' not in payload:
                raise AssertionError("Invalid RPC Call.")
            return self.rpc_call(request, **payload)

        except Exception as e: # noqa (any error)
            return SerializedHttpResponse(dict(error=dict(message=str(e))), error=True)

    def rpc_call(self, request, method=None, params=None, **kwargs):
        """ Call a RPC method.

        return object: a result

        """
        args = []
        kwargs = dict()
        if isinstance(params, dict):
            kwargs.update(params)
        else:
            args = list(as_tuple(params))

        method_key = "{0}.{1}".format(self.scheme_name, method)
        if method_key not in self.methods:
            raise AssertionError("Unknown method: {0}".format(method))
        method = self.methods[method_key]

        if hasattr(method, 'request'):
            args.insert(0, request)

        return method(*args, **kwargs)


class AutoJSONRPC(RPCResource):

    """ Automatic JSONRPC Api from REST.

    Automatic Implementation of remote procedure call based on your REST.

    """

    separator = '.'

    class Meta:
        url_name = 'autojsonrpc'

    @staticmethod
    def configure_rpc(scheme=None):
        """ Do nothing. """
        pass

    def rpc_call(self, request, method=None, **payload):
        """ Call REST API with RPC force.

        return object: a result

        """
        if not method or self.separator not in method:
            raise AssertionError("Wrong method name: {0}".format(method))

        resource_name, method = method.split(self.separator, 1)
        if resource_name not in self.api.resources:
            raise AssertionError("Unknown method " + method)

        data = QueryDict('', mutable=True)
        data.update(payload.get('data', dict()))
        data['callback'] = payload.get('callback') or request.GET.get(
            'callback') or request.GET.get('jsonp') or 'callback'
        for h, v in payload.get('headers', dict()).iteritems():
            request.META["HTTP_%s" % h.upper().replace('-', '_')] = v

        request.POST = request.PUT = request.GET = data
        delattr(request, '_request')
        request.method = method.upper()
        request.META['CONTENT_TYPE'] = 'application/x-www-form-urlencoded'
        params = payload.pop('params', dict())
        response = self.api.call(resource_name, request, **params)

        if not isinstance(response, SerializedHttpResponse):
            return response

        if response['Content-type'] in self._meta.emitters_dict:
            return HttpResponse(response.content, status=response.status_code)

        if response.status_code == 200:
            return response.response

        raise AssertionError(response.response)


# pymode:lint_ignore=E1103,W0703



""" Collect URLS from apps. """
from django.conf.urls import include, patterns

from ..main.api import API as main
from ..main.resources import DummyResource
from ..rpc.api import API as rpc
from .api import api as pirates, api2 as pirates2


urlpatterns = main.urls + patterns(
    '',
    DummyResource.as_url(),
    (r'^rpc/', include(rpc.urls)),

    (r'^pirates/', include(pirates.urls)),
    (r'^pirates2/', include(pirates2.urls)),

)



""" Proxy request to Google Analytics. """
from pyga.requests import Tracker, Visitor, Session, Page

from ..views import ResourceView


__all__ = 'GaResource',


class GaResource(ResourceView):

    """ Google Analytics support.

    Track GA from server.

    Example: ::

        api.register(GaResource, account_id="UA-123456", domain="test")

    """

    class Meta:
        url_regex = r'^ga(?P<path>.*)$'
        domain = None
        account_id = None

    def get(self, request, path=None, **resources):
        """ Proxy request to GA. """
        tracker = Tracker(
            self._meta.account_id,
            self._meta.domain or request.META.get('SERVER_NAME'))
        visitor = Visitor()
        visitor.extract_from_server_meta(request.META)
        session = Session()
        page = Page(path)
        tracker.track_pageview(page, session, visitor)

# lint_ignore=F0401



""" Generate a resource's map. """
from django.forms.models import ModelChoiceField
from django.utils.encoding import smart_unicode

from ..settings import ADREST_MAP_TEMPLATE
from ..utils.auth import AnonimousAuthenticator
from ..utils.emitter import HTMLTemplateEmitter, JSONEmitter
from ..views import ResourceView


__all__ = 'MapResource',


class MapResource(ResourceView):

    """ Simple Api Map. """

    class Meta:
        authenticators = AnonimousAuthenticator
        emit_template = ADREST_MAP_TEMPLATE
        emitters = HTMLTemplateEmitter, JSONEmitter
        log = False
        url_regex = r'^map$'

    def get(self, *args, **Kwargs):
        """ Render map.

        :return list: list of resources.

        """
        return list(self.__gen_apimap())

    def __gen_apimap(self):
        for url_name in sorted(self.api.resources.iterkeys()):
            resource = self.api.resources[url_name]
            info = dict(
                resource=resource,
                url_name=resource._meta.url_name,
                allowed_methods=resource._meta.allowed_methods,
                doc=resource.__doc__,
                emitters=', '.join(
                    [e.media_type for e in resource._meta.emitters]),
                fields=[],
                model=None,
            )
            if resource._meta.model:
                info['model'] = dict(
                    name="{0}.{1}".format(
                        resource._meta.model._meta.module_name, # nolint
                        resource._meta.model._meta.object_name, # nolint
                    ),
                    fields=resource._meta.model._meta.fields # nolint
                )

            models = [
                p._meta.model for p in resource._meta.parents if p._meta.model]

            if resource._meta.form and (
                'POST' in resource._meta.allowed_methods
                    or 'PUT' in resource._meta.allowed_methods):
                info['fields'] += [
                    (name, dict(
                        required=f.required and f.initial is None,
                        label=f.label,
                        help=smart_unicode(f.help_text + ''))
                     )
                    for name, f in resource._meta.form.base_fields.iteritems()
                    if not (isinstance(f, ModelChoiceField)
                            and f.choices.queryset.model in models)
                ]

            for a in resource._meta.authenticators:
                info['fields'] += a.get_fields()

            info['auth'] = set(
                a.__doc__ or 'Custom' for a in resource._meta.authenticators)

            key = resource._meta.url_regex\
                .replace("(?P", "")\
                .replace("[^/]+)", "")\
                .replace("?:", "")\
                .replace("$", "")\
                .replace("^", "/")

            if getattr(resource, "methods", None):
                import inspect
                info['methods'] = dict()
                for name, method in resource.methods.items():
                    info['methods'][name] = inspect.getargspec(method)
            yield key, info

# lint_ignore=W0212



""" Meta support for ADRest classes.
"""
from django.db.models import get_model, Model


__all__ = 'MixinBaseMeta', 'MixinBase'


class MetaOptions(dict):

    """ Storage for Meta options. """

    def __getattr__(self, name):
        return self.get(name)

    __setattr__ = dict.__setitem__


class Meta:

    """ Base options for all ADRest mixins.

    With Meta options you can setup your resources.

    """

    # Link to parent resource. Used for create a resource's hierarchy.
    parent = None

    #: Setup Django ORM model.
    #: Value could be a model class or string path like 'app.model'.
    model = None


class MixinBaseMeta(type):

    """ Init meta options.

    Merge Meta options from class bases.

    """

    def __new__(mcs, name, bases, params):
        params['_meta'] = params.get('_meta', MetaOptions())
        cls = super(MixinBaseMeta, mcs).__new__(mcs, name, bases, params)

        meta = dict()
        for parent in reversed(cls.mro()):
            if hasattr(parent, 'Meta'):
                meta.update(parent.Meta.__dict__)

        cls._meta.update(dict(
            (attr, meta[attr]) for attr in meta
            if not attr.startswith('_')
        ))

        # Prepare hierarchy
        cls._meta.parents = []
        if cls._meta.parent:
            cls._meta.parents = (
                cls._meta.parent._meta.parents + [cls._meta.parent])

        if not cls._meta.model:
            return cls

        # Create model from string
        if isinstance(cls._meta.model, basestring):
            if not '.' in cls._meta.model:
                raise AssertionError(
                    "'model_class' must be either a model"
                    " or a model name in the format"
                    " app_label.model_name")
            cls._meta.model = get_model(*cls._meta.model.split("."))

        # Check meta.name and queryset
        if not issubclass(cls._meta.model, Model):
            raise AssertionError("'model' attribute must be subclass of Model")

        cls._meta.fields = set(f.name for f in cls._meta.model._meta.fields)

        return cls


class MixinBase(object):

    """ Base class for all ADRest mixins.

    .. autoclass:: adrest.utils.meta.Meta
       :members:

    """

    Meta = Meta

    __metaclass__ = MixinBaseMeta

    __parent__ = None

    @property
    def parent(self):
        """ Cache a instance of self parent class.

        :return object: instance of self.Meta.parent class

        """
        if not self._meta.parent:
            return None

        if not self.__parent__:
            self.__parent__ = self._meta.parent()

        return self.__parent__



""" ADRest mail utils. """
import traceback
import sys
from django.core.mail import mail_admins
from ..settings import ADREST_MAIL_ERRORS


def adrest_errors_mail(response, request):
    """ Send a mail about ADRest errors.

    :return bool: status of operation

    """

    if not response.status_code in ADREST_MAIL_ERRORS:
        return False

    subject = 'ADREST API Error (%s): %s' % (
        response.status_code, request.path)
    stack_trace = '\n'.join(traceback.format_exception(*sys.exc_info()))
    message = """
Stacktrace:
===========
%s

Handler data:
=============
%s

Request information:
====================
%s

""" % (stack_trace, repr(getattr(request, 'data', None)), repr(request))
    return mail_admins(subject, message, fail_silently=True)



""" Testing settings.
"""
from .production import * # nolint

ENVIRONMENT_NAME = 'test'

# Databases
DATABASES['default']['ENGINE'] = 'django.db.backends.sqlite3'
DATABASES['default']['NAME'] = ':memory:'

# Caches
CACHES['default']['BACKEND'] = 'django.core.cache.backends.locmem.LocMemCache'
CACHES['default']['KEY_PREFIX'] = '_'.join((PROJECT_NAME, 'TST'))

# Disable south migrations on db creation in tests
SOUTH_TESTS_MIGRATE = False

# CELERY
BROKER_BACKEND = 'memory'
CELERY_ALWAYS_EAGER = True

logging.info('Test settings are loaded.')



""" Create Django project for testing. """

from django.conf import settings

# Configure Django
settings.configure(
    ADMINS=('test', 'test@test.com'),

    ROOT_URLCONF='tests.core.urls',
    DEBUG=True,

    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': ':memory:',
            'USER': '',
            'PASSWORD': '',
            'TEST_CHARSET': 'utf8',
        }
    },
    CACHE_BACKEND='locmem://',

    INSTALLED_APPS=(
        'django.contrib.contenttypes',
        'django.contrib.auth',
        'adrest',
        'tests.core', 'tests.main', 'tests.rpc',
    ),

    TEMPLATE_DEBUG=True,
    TEMPLATE_CONTEXT_PROCESSORS=(
        'django.core.context_processors.static',
        'django.core.context_processors.request',
        'django.contrib.auth.context_processors.auth',
    ),

    EMAIL_BACKEND='django.core.mail.backends.locmem.EmailBackend',

    ADREST_ACCESS_LOG=True,
    ADREST_ALLOW_OPTIONS=True,
    ADREST_MAIL_ERRORS=(500, 400),
    ADREST_AUTO_CREATE_ACCESSKEY=True,
)

# Django 1.7
from django import VERSION

if VERSION >= (1, 7):
    from django.apps import apps

    apps.populate(settings.INSTALLED_APPS)


# Setup tests
from django.core.management import call_command
call_command('syncdb', interactive=False)

from .core.tests import * # noqa
from .main.tests import * # noqa
from .rpc.tests import * # noqa

# lint_ignore=W0614,W0401,E272



""" ADRest emitters. """

from datetime import datetime
from os import path as op
from time import mktime

from django.db.models.base import ModelBase, Model
from django.template import RequestContext, loader

from ..utils import UpdatedList
from .paginator import Paginator
from .response import SerializedHttpResponse
from .serializer import JSONSerializer, XMLSerializer
from .status import HTTP_200_OK


__all__ = (
    'NullEmitter', 'TextEmitter', 'JSONEmitter', 'JSONPEmitter', 'XMLEmitter',
    'HTMLTemplateEmitter', 'XMLTemplateEmitter', 'BaseEmitter')


class EmitterMeta(type):

    """ Preload format attribute. """

    def __new__(mcs, name, bases, params):
        cls = super(EmitterMeta, mcs).__new__(mcs, name, bases, params)
        if not cls.format and cls.media_type:
            cls.format = str(cls.media_type).split('/')[-1]
        return cls


class BaseEmitter(object):

    """ Base class for emitters.

    All emitters must extend this class, set the media_type attribute, and
    override the serialize() function.

    """

    __metaclass__ = EmitterMeta

    media_type = None
    format = None

    def __init__(self, resource, request=None, response=None):
        self.resource = resource
        self.request = request
        self.response = SerializedHttpResponse(
            response, content_type=self.media_type, status=HTTP_200_OK)

    def emit(self):
        """ Serialize response.

        :return response: Instance of django.http.Response

        """
        # Skip serialize
        if not isinstance(self.response, SerializedHttpResponse):
            return self.response

        self.response.content = self.serialize(self.response.response)
        self.response['Content-type'] = self.media_type
        return self.response

    @staticmethod
    def serialize(content):
        """ Low level serialization.

        :return response:

        """
        return content


class NullEmitter(BaseEmitter):

    """ Return data as is. """

    media_type = 'unknown/unknown'

    def emit(self):
        """ Do nothing.

        :return response:

        """
        return self.response


class TextEmitter(BaseEmitter):

    """ Serialize to unicode. """

    media_type = 'text/plain'

    @staticmethod
    def serialize(content):
        """ Get content and return string.

        :return unicode:

        """
        return unicode(content)


class JSONEmitter(BaseEmitter):

    """ Serialize to JSON. """

    media_type = 'application/json'

    def serialize(self, content):
        """ Serialize to JSON.

        :return string: serializaed JSON

        """
        worker = JSONSerializer(
            scheme=self.resource,
            options=self.resource._meta.emit_options,
            format=self.resource._meta.emit_format,
            **self.resource._meta.emit_models
        )
        return worker.serialize(content)


class JSONPEmitter(JSONEmitter):

    """ Serialize to JSONP. """

    media_type = 'text/javascript'

    def serialize(self, content):
        """ Serialize to JSONP.

        :return string: serializaed JSONP

        """
        content = super(JSONPEmitter, self).serialize(content)
        callback = self.request.GET.get('callback', 'callback')
        return u'%s(%s)' % (callback, content)


class XMLEmitter(BaseEmitter):

    """ Serialize to XML. """

    media_type = 'application/xml'
    xmldoc_tpl = '<?xml version="1.0" encoding="utf-8"?>\n<response success="%s" version="%s" timestamp="%s">%s</response>' # noqa

    def serialize(self, content):
        """ Serialize to XML.

        :return string: serialized XML

        """
        worker = XMLSerializer(
            scheme=self.resource,
            format=self.resource._meta.emit_format,
            options=self.resource._meta.emit_options,
            **self.resource._meta.emit_models
        )
        return self.xmldoc_tpl % (
            'true' if not self.response.error else 'false',
            str(self.resource.api or ''),
            int(mktime(datetime.now().timetuple())),
            worker.serialize(content)
        )


class TemplateEmitter(BaseEmitter):

    """ Serialize by django templates. """

    def serialize(self, content):
        """ Render Django template.

        :return string: rendered content

        """
        if self.response.error:
            template_name = op.join('api', 'error.%s' % self.format)
        else:
            template_name = (self.resource._meta.emit_template
                             or self.get_template_path(content))

        template = loader.get_template(template_name)

        return template.render(RequestContext(self.request, dict(
            content=content,
            emitter=self,
            resource=self.resource)))

    def get_template_path(self, content=None):
        """ Find template.

        :return string: remplate path

        """
        if isinstance(content, Paginator):
            return op.join('api', 'paginator.%s' % self.format)

        if isinstance(content, UpdatedList):
            return op.join('api', 'updated.%s' % self.format)

        app = ''
        name = self.resource._meta.name

        if not content:
            content = self.resource._meta.model

        if isinstance(content, (Model, ModelBase)):
            app = content._meta.app_label
            name = content._meta.module_name

        basedir = 'api'
        if getattr(self.resource, 'api', None):
            basedir = self.resource.api.prefix

        return op.join(
            basedir,
            str(self.resource.api or ''), app, "%s.%s" % (name, self.format)
        )


class JSONTemplateEmitter(TemplateEmitter):

    """ Template emitter with JSON media type. """

    media_type = 'application/json'


class JSONPTemplateEmitter(TemplateEmitter):

    """ Template emitter with javascript media type. """

    media_type = 'text/javascript'
    format = 'json'

    def serialize(self, content):
        """ Move rendered content to callback.

        :return string: JSONP

        """
        content = super(JSONPTemplateEmitter, self).serialize(content)
        callback = self.request.GET.get('callback', 'callback')
        return '%s(%s)' % (callback, content)


class HTMLTemplateEmitter(TemplateEmitter):

    """ Template emitter with HTML media type. """

    media_type = 'text/html'


class XMLTemplateEmitter(TemplateEmitter):

    """ Template emitter with XML media type. """

    media_type = 'application/xml'
    xmldoc_tpl = '<?xml version="1.0" encoding="utf-8"?>\n<response success="%s" version="%s" timestamp="%s">%s</response>' # noqa

    def serialize(self, content):
        """ Serialize to xml.

        :return string:

        """
        return self.xmldoc_tpl % (
            'true' if self.response.status_code == HTTP_200_OK else 'false',
            str(self.resource.api or ''),
            int(mktime(datetime.now().timetuple())),
            super(XMLTemplateEmitter, self).serialize(content)
        )


# pymode:lint_ignore=F0401,W0704



""" Development related settings.
"""
from .production import * # nolint
from .core import TEMPLATE_LOADERS

assert TEMPLATE_LOADERS

ENVIRONMENT_NAME = 'development'

# Caches
CACHES['default']['KEY_PREFIX'] = '_'.join((PROJECT_NAME, 'DEV'))

# Debug
DEBUG = True
TEMPLATE_DEBUG = True
if DEBUG:
    INTERNAL_IPS += tuple('192.168.1.%s' % x for x in range(1, 111))
    TEMPLATE_CONTEXT_PROCESSORS += 'django.core.context_processors.debug',
    MIDDLEWARE_CLASSES += (
        'debug_toolbar.middleware.DebugToolbarMiddleware', )
    INSTALLED_APPS += ('debug_toolbar', )
    DEBUG_TOOLBAR_CONFIG = dict(INTERCEPT_REDIRECTS=False)

# Logging
LOGGING['loggers']['django.request']['level'] = 'DEBUG'
LOGGING['loggers']['celery']['level'] = 'DEBUG'
logging.info('Development settings are loaded.')



""" Simple API for tests. """
from adrest.views import ResourceView
from adrest.api import Api


api = Api(api_rpc=True)


@api.register
class PirateResource(ResourceView):

    """ Part of simple API for tests. """

    class Meta:
        allowed_methods = 'get', 'POST', 'pUt', 'delete', 'Patch'
        model = 'core.pirate'


@api.register
class BoatResource(ResourceView):

    """ Part of simple API for tests. """

    class Meta:
        allowed_methods = 'get', 'post', 'put', 'delete'
        model = 'core.boat'
        parent = PirateResource


api2 = Api('1.0.0')
api2.register(PirateResource)



""" Filters and sorting support. """
from django.core.exceptions import FieldError
from logging import getLogger

from ..settings import ADREST_LIMIT_PER_PAGE
from ..utils import UpdatedList
from ..utils.meta import MixinBaseMeta, MixinBase
from ..utils.paginator import Paginator


logger = getLogger('django.request')

# Separator used to split filter strings apart.
LOOKUP_SEP = '__'


class Meta:

    """ Options for dynamic mixin.

    Setup parameters for filtering and sorting a resources.

    ::
        class SomeResource(DynamicMixin, View):

            class Meta:
                dyn_prefix = 'dyn-'

    """

    #: Prefix for dynamic fields
    dyn_prefix = 'adr-'

    #: Limit per page for pagination
    #: Set to `0` for disable pagination in resource, but user can still force
    #: it with `?max=...`
    limit_per_page = ADREST_LIMIT_PER_PAGE

    #: Define queryset for resource's operation.
    #: By default: self.Meta.model.objects.all()
    queryset = None


class DynamicMixinMeta(MixinBaseMeta):

    """ Prepare dynamic class. """

    def __new__(mcs, name, bases, params):

        cls = super(DynamicMixinMeta, mcs).__new__(mcs, name, bases, params)

        if not cls._meta.dyn_prefix:
            raise AssertionError("Resource.Meta.dyn_prefix should be defined.")

        if cls._meta.model and cls._meta.queryset is None:
            cls._meta.queryset = cls._meta.model.objects.all()

        return cls


class DynamicMixin(MixinBase):

    """ Implement filters and sorting.

    ADRest DynamicMixin supports filtering and sorting collection from query
    params.

    """

    __metaclass__ = DynamicMixinMeta

    Meta = Meta

    def __init__(self, *args, **kwargs):
        """ Copy self queryset for prevent query caching. """

        super(DynamicMixin, self).__init__(*args, **kwargs)

        if not self._meta.queryset is None:
            self._meta.queryset = self._meta.queryset.all()

    def get_collection(self, request, **resources):
        """ Get filters and return filtered result.

        :return collection: collection of related resources.

        """

        if self._meta.queryset is None:
            return []

        # Filter collection
        filters = self.get_filters(request, **resources)
        filters.update(self.get_default_filters(**resources))
        qs = self._meta.queryset
        for key, (value, exclude) in filters.items():
            try:
                if exclude:
                    qs = qs.exclude(**{key: value})

                else:
                    qs = qs.filter(**{key: value})
            except FieldError, e:
                logger.warning(e)

        sorting = self.get_sorting(request, **resources)
        if sorting:
            qs = qs.order_by(*sorting)

        return qs

    def get_default_filters(self, **resources):
        """ Return default filters by a model fields.

        :return dict: name, field

        """
        return dict((k, (v, False)) for k, v in resources.items()
                    if k in self._meta.fields)

    def get_filters(self, request, **resources):
        """ Make filters from GET variables.

        :return dict: filters

        """
        filters = dict()

        if not self._meta.fields:
            return filters

        for field in request.GET.iterkeys():
            tokens = field.split(LOOKUP_SEP)
            field_name = tokens[0]

            if not field_name in self._meta.fields:
                continue

            exclude = False
            if tokens[-1] == 'not':
                exclude = True
                tokens.pop()

            converter = self._meta.model._meta.get_field(
                field_name).to_python if len(tokens) == 1 else lambda v: v
            value = map(converter, request.GET.getlist(field))

            if len(value) > 1:
                tokens.append('in')
            else:
                value = value.pop()

            filters[LOOKUP_SEP.join(tokens)] = (value, exclude)

        return filters

    def get_sorting(self, request, **resources):
        """ Get sorting options.

        :return list: sorting order

        """
        sorting = []

        if not request.GET:
            return sorting

        prefix = self._meta.dyn_prefix + 'sort'
        return request.GET.getlist(prefix)

    def paginate(self, request, collection):
        """ Paginate collection.

        :return object: Collection or paginator

        """
        p = Paginator(request, self, collection)
        return p.paginator and p or UpdatedList(collection)



from adrest.resources.rpc import get_request


def method1(name):
    return "Hello {0}".format(name)


def method2(start=1, end=100):
    from random import randint
    return randint(start, end)


def error_method():
    raise Exception('Error here')


@get_request
def method3(request, name):
    return request.method + name


def __private_method():
    raise Exception("I am hidden!")



""" Create Api and resources. """
from adrest.api import Api
from adrest.views import ResourceView
from adrest.resources.rpc import RPCResource

from . import rpc


api = Api(version='0.1')
api.register(RPCResource, scheme=rpc)


@api.register
class AuthorResource(ResourceView):

    """ Get authors from db. """

    class Meta:
        model = 'main.author'


@api.register
class BookResource(ResourceView):

    """ Works with books. Nested resource. """

    class Meta:
        allowed_methods = 'get', 'post', 'put'
        model = 'main.book'
        parent = AuthorResource



""" ADRest test's helpers. """
import json as js
from django.test import TestCase
from django.utils.functional import curry

from .client import AdrestClient, reverse


class AdrestTestCase(TestCase):

    """ TestCase for ADRest related tests. """

    api = None
    client_class = AdrestClient

    @classmethod
    def reverse(cls, resource, **resources):
        """ Reverse resource by ResourceClass or name.

        :param resource: Resource Class or String name.
        :param **resources: Uri params

        :return str: URI string

        """
        if not cls.api:
            raise AssertionError("AdrestTestCase must have the api attribute.")

        return reverse(resource, cls.api, **resources)

    def get_resource(self, resource, **kwargs):
        """ Simply run resource method.

        :param resource: Resource Class or String name.
        :param data: Request data
        :param json: Make JSON request
        :param headers: Request headers
        :param key: HTTP_AUTHORIZATION token

        :return object: result

        """
        return self.client.request_resource(resource, api=self.api, **kwargs)

    def rpc(self, resource, rpc=None, headers=None, callback=None, key=None,
            **kwargs):
        """ Emulate RPC call.

        :param resource: Resource Class or String name.
        :param rpc: RPC params.
        :param headers: Send headers
        :param callback: JSONP callback

        :return object: result

        """
        data = rpc or dict()
        headers = headers or dict()
        headers['content_type'] = 'application/json'

        if callback:
            headers['HTTP_ACCEPT'] = 'text/javascript'
            method = 'get'
            data = dict(
                callback=callback,
                payload=js.dumps(data))

        else:
            headers['HTTP_ACCEPT'] = 'application/json'
            method = 'post'
            data = js.dumps(data)

        return self.client.request_resource(
            resource, method=method, data=data, headers=headers, api=self.api, key=key, **kwargs)

    put_resource = curry(get_resource, method='put')
    post_resource = curry(get_resource, method='post')
    patch_resource = curry(get_resource, method='patch')
    delete_resource = curry(get_resource, method='delete')



""" Tests ADRest emitter mixin.
"""
from django.views.generic import View

from ..api import api as API
from adrest.mixin import EmitterMixin
from adrest.tests import AdrestTestCase
from mixer.backend.django import mixer


class CoreEmitterTest(AdrestTestCase):

    """ Emitter related tests. """

    api = API

    def test_meta(self):
        """ Test a meta attribute generation. """

        class Resource(View, EmitterMixin):

            class Meta:
                model = 'core.pirate'

        self.assertTrue(Resource._meta)
        self.assertTrue(Resource._meta.emitters)

    def test_to_simple(self):
        """ Test resource's to simple method.

        :return :

        """

        pirates = mixer.cycle(2).blend('core.pirate')

        class Resource(View, EmitterMixin):

            class Meta:
                model = 'core.pirate'

            def to_simple(self, content, simple, serializer=None):

                return simple + ['HeyHey!']

        resource = Resource()
        response = resource.emit(pirates)
        self.assertTrue('HeyHey!' in response.content)

        class Resource(View, EmitterMixin):

            class Meta:
                model = 'core.pirate'

            @staticmethod
            def to_simple__name(pirate, serializer=None):

                return 'Evil ' + pirate.name

        resource = Resource()
        pirate = pirates[0]
        response = resource.emit(pirate)
        self.assertTrue('Evil ' + pirate.name in response.content)

    def test_model_options(self):

        boats = mixer.cycle(2).blend('core.boat')

        class Resource(View, EmitterMixin):

            class Meta:
                model = 'core.boat'
                emit_models = dict(
                    include='hooray'
                )

            @staticmethod
            def to_simple__hooray(boat, serializer=None):
                return 'hooray'

        resource = Resource()
        response = resource.emit(boats)
        self.assertTrue('hooray' in response.content)

        boat = boats[0]
        resource._meta.emit_models['exclude'] = 'title'
        response = resource.emit(boat)
        self.assertFalse(boat.title in response.content)

        resource._meta.emit_models['related'] = dict(
            pirate=dict(
                fields='name'
            )
        )
        response = resource.emit(boat)
        self.assertTrue(boat.pirate.name in response.content)

        class Resource(View, EmitterMixin):

            class Meta:
                model = 'core.boat'
                emit_include = 'hooray'

            @staticmethod
            def to_simple__hooray(boat, serializer=None):
                return 'hooray'

        resource = Resource()
        response = resource.emit(boats)
        self.assertTrue('hooray' in response.content)

    def test_format(self):
        pirate = mixer.blend('core.pirate')

        class Resource(View, EmitterMixin):
            class Meta:
                model = 'core.pirate'

        resource = Resource()
        response = resource.emit(pirate)
        self.assertTrue('fields' in response.content)

        resource._meta.emit_format = 'simple'
        response = resource.emit(pirate)
        self.assertFalse('fields' in response.content)


# lint_ignore=W0212,E0102,C0110



""" Collect tests.
"""
from .admin import *
from .api import *
from .auth import *
from .dynamic import *
from .emitter import *
from .handler import *
from .resource import *
from .rpc import *
from .serializer import *
from .test import *
from .utils import *


# lint_ignore=W0401



import random
import re
from decimal import Decimal

from django.contrib.auth.models import User
from django.core import mail
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.test import TestCase, Client, RequestFactory
from django.views.generic import View
from mixer.backend.django import mixer

from .api import API as api
from .models import Author, Book
from .resources import (AuthorResource, BookPrefixResource,
                        ArticleResource, SomeOtherResource, BookResource)
from adrest.mixin.emitter import EmitterMixin
from adrest.models import Access
from adrest.tests.utils import AdrestTestCase
from adrest.utils import emitter, parser


class MixinTest(TestCase):

    def setUp(self):
        self.rf = RequestFactory()

    def test_emitter(self):
        request = self.rf.get("/")

        class Test(View, EmitterMixin):

            def get(self, request):
                content = self.emit('test')
                response = HttpResponse(content)
                return response

        test = Test()
        response = test.dispatch(request)
        self.assertContains(response, 'test')

        content = emitter.JSONSerializer().serialize([1, Decimal('3.4'), None])
        self.assertTrue('3.' in content and 'null' in content)


class MetaTest(TestCase):

    def test_meta(self):
        self.assertTrue(AuthorResource._meta)
        self.assertEqual(AuthorResource._meta.allowed_methods, (
            'GET', 'POST', 'PATCH', 'OPTIONS', 'HEAD'
        ))
        self.assertEqual(AuthorResource._meta.name, 'author')
        self.assertEqual(AuthorResource._meta.url_name, 'author')
        self.assertEqual(AuthorResource._meta.url_regex, '^owner/$')
        self.assertEqual(AuthorResource._meta.parents, [])
        self.assertEqual(AuthorResource._meta.emitters_dict, {
            emitter.JSONEmitter.media_type: emitter.JSONEmitter,
        })
        self.assertEqual(AuthorResource._meta.parsers_dict, {
            parser.FormParser.media_type: parser.FormParser,
            parser.XMLParser.media_type: parser.XMLParser,
            parser.JSONParser.media_type: parser.JSONParser,
        })
        self.assertEqual(
            AuthorResource._meta.default_parser, parser.FormParser)

    def test_meta_parents(self):
        self.assertEqual(AuthorResource._meta.parents, [])
        self.assertEqual(BookPrefixResource._meta.parents, [AuthorResource])
        self.assertEqual(
            ArticleResource._meta.parents,
            [AuthorResource, BookPrefixResource])

    def test_meta_name(self):
        self.assertEqual(AuthorResource._meta.name, 'author')
        self.assertEqual(BookPrefixResource._meta.name, 'book')
        self.assertEqual(SomeOtherResource._meta.name, 'someother')

    def test_meta_url_name(self):
        self.assertEqual(AuthorResource._meta.url_name, 'author')
        self.assertEqual(BookResource._meta.url_name, 'author-book')
        self.assertEqual(BookPrefixResource._meta.url_name, 'author-test-book')
        self.assertEqual(
            ArticleResource._meta.url_name, 'author-test-book-article')
        self.assertEqual(
            SomeOtherResource._meta.url_name, 'author-device-someother')

    def test_meta_url_regex(self):
        self.assertEqual(AuthorResource._meta.url_regex, '^owner/$')
        self.assertEqual(BookPrefixResource._meta.url_regex,
                         'owner/test/book/(?P<book>[^/]+)?')

        self.assertEqual(
            ArticleResource._meta.url_regex,
            'owner/test/book/(?P<book>[^/]+)?/article/(?P<article>[^/]+)?')

        self.assertEqual(
            SomeOtherResource._meta.url_regex,
            'owner/device/(?P<device>[^/]+)/someother/(?P<someother>[^/]+)?')


class ApiTest(AdrestTestCase):

    api = api

    def test_api(self):
        self.assertTrue(api.version)
        self.assertEqual(str(api), "1.0.0")
        self.assertTrue(api.urls)

    def test_urls(self):
        uri = self.reverse('test')
        self.assertEqual(uri, '/1.0.0/test/mem')
        response = self.client.get('/1.0.0/test/mem')
        self.assertContains(response, 'true')

        response = self.client.get('/1.0.0/test/mem/')
        self.assertContains(response, 'true')


class AdrestTest(AdrestTestCase):

    api = api

    def setUp(self):
        self.author = mixer.blend('main.author')
        self.book = mixer.blend('main.book', author=self.author)

    def test_urls(self):
        uri = reverse('iamdummy')
        self.assertEqual(uri, '/iamdummy/')

    def test_methods(self):
        uri = self.reverse('author')
        self.assertEqual(uri, '/1.0.0/owner')
        response = self.client.get(uri)
        self.assertContains(response, 'true')

        response = self.client.put(uri)
        self.assertContains(response, 'false', status_code=405)

        response = self.client.head(uri)
        self.assertEqual(response.status_code, 200)

    def test_owners_checking(self):
        response = self.get_resource(
            'author-test-book-article', book=self.book.pk, data=dict(
                author=self.author.pk))
        self.assertContains(response, 'false', status_code=401)

        response = self.get_resource(
            'author-test-book-article',
            key=self.author.user.accesskey_set.get(),
            book=self.book.pk, data=dict(author=self.author.pk))
        self.assertContains(response, 'true')

    def test_access_logging(self):
        uri = self.reverse('author-test-book-article', book=self.book.pk)
        self.client.get(uri)
        access = Access.objects.get()
        self.assertEqual(access.uri, uri)
        self.assertEqual(access.version, str(api))

        # Do not write to access log
        response = self.get_resource('csv')
        self.assertEquals(response['Content-Type'], 'text/csv')
        access = Access.objects.filter(uri=response.request['PATH_INFO'])
        self.assertEquals(access.count(), 1)
        self.assertEquals(
            access.get().response, "Invalid response content encoding")

    def test_options(self):
        self.assertTrue('OPTIONS' in ArticleResource._meta.allowed_methods)
        uri = self.reverse('author-test-book-article', book=self.book.pk)
        response = self.client.options(uri, data=dict(author=self.author.pk))
        self.assertContains(response, 'OK')

        author = mixer.blend('main.author')
        response = self.client.options(uri, data=dict(author=author.pk))
        self.assertContains(response, 'OK')


class ResourceTest(AdrestTestCase):

    api = api

    def setUp(self):
        super(ResourceTest, self).setUp()
        for i in range(5):
            user = User.objects.create(username='test%s' % i)
            self.author = Author.objects.create(name='author%s' % i, user=user)

        for i in range(148):
            Book.objects.create(author=self.author, title="book%s" %
                                i, status=random.choice((1, 2, 3)), price=432)

    def test_patch(self):
        response = self.patch_resource('author')
        self.assertContains(response, 'found', status_code=404)

    def test_author(self):
        response = self.get_resource('author')
        self.assertContains(response, 'count="5"')

        response = self.post_resource('author', data=dict(
            name="new author",
            user=User.objects.create(username="new user").pk))
        self.assertContains(response, 'new author')

        response = self.post_resource('author', data=dict(name="author 22"))
        self.assertContains(response, 'field is required', status_code=400)

    def test_collection_put_delete(self):
        status1 = Book.objects.filter(status=1)
        response = self.put_resource('author-test-book', data=dict(
            status=3,
            author=self.author.pk,
            book=[b.pk for b in status1]
        ))
        self.assertContains(response, 'count="%s"' % len(status1))
        self.assertContains(response, '<status>3</status>')
        self.assertNotContains(response, '<status>1</status>')
        self.assertFalse(Book.objects.filter(status=1).count())

        status2 = Book.objects.filter(status=2)
        response = self.delete_resource('author-test-book', data=dict(
            author=self.author.pk,
            book=[b.pk for b in status2]
        ))
        self.assertContains(response, '')
        self.assertFalse(Book.objects.filter(status=2).count())

    def test_book(self):
        uri = self.reverse('author-test-book')
        self.assertEqual(uri, "/1.0.0/owner/test/book/")

        response = self.get_resource('author-test-book', data=dict(
            author=self.author.pk
        ))
        self.assertContains(response, 'count="%s"' %
                            Book.objects.filter(author=self.author).count())
        self.assertContains(response, '<name>%s</name>' % self.author.name)
        self.assertContains(response, '<book_price>432</book_price>')

        response = self.get_resource('author-test-book', data=dict(
            title__startswith="book1",
            title__megadeath=12,
        ))
        self.assertContains(response, 'count="%s"' % Book.objects.filter(
            title__startswith='book1').count())

        response = self.post_resource('author-test-book', data=dict(
            title="new book",
            status=2,
            author=self.author.pk))
        self.assertContains(response, '<price>0</price>')
        self.assertContains(response, '<json>{"')  # noqa

        response = self.post_resource('author-test-book', json=True, data=dict(
            title="new book",
            status=2,
            author=self.author.pk))
        self.assertContains(response, '<price>0</price>')

        uri = self.reverse('author-test-book', book=1)
        uri = "%s?author=%s" % (uri, self.author.pk)
        response = self.client.put(uri, data=dict(
            price=199
        ))
        self.assertContains(response, '<price>199</price>')

        response = self.client.delete(uri)
        self.assertContains(response, '')

    def test_filter(self):
        uri = self.reverse('author-test-book')
        response = self.client.get(uri, data=dict(
            author=self.author.pk,
            title="book2"))
        self.assertContains(response, 'count="1"')

        response = self.client.get(uri, data=dict(
            author=self.author.pk,
            status=[1, 2, 3]))
        self.assertContains(
            response, 'count="%s"' % Book.objects.all().count())

        response = self.client.get(uri, data=dict(
            author=self.author.pk,
            status=[1, 3]))
        self.assertNotContains(response, '<status>2</status>')

        response = self.client.get(
            uri + "?title=book2&title=book3&author=%s" % self.author.pk)
        self.assertContains(response, 'count="2"')

    def test_not_filter(self):
        uri = self.reverse('author-test-book')

        exclude_author = Author.objects.create(
            name="exclude_author",
            user=User.objects.create(username="exclude_user"))

        for i in xrange(5):
            Book.objects.create(
                author=exclude_author, title="book_for_exclude%s" % i,
                status=i % 3 + 1, price=482)

        response = self.client.get(uri, data=dict(
            author__not=self.author.pk))

        self.assertContains(response, '<results count="5" page="1">')
        self.assertContains(response, '<name>exclude_author</name>')

        for i in xrange(5):
            self.assertContains(
                response, '<title>book_for_exclude%s</title>' % i)

        response = self.client.get(uri, data=dict(
            author__not=self.author.pk,
            status__not=[1, 3]))

        self.assertContains(response, '<results count="%s" page="1">' %
                            Book.objects.filter(author=exclude_author).
                            exclude(status__in=[1, 3]).count())
        self.assertNotContains(response, '<status>3</status>')
        self.assertNotContains(response, '<status>1</status>')
        self.assertContains(response, '<status>2</status>')
        self.assertContains(response, '<name>exclude_author</name>')

    def test_custom(self):
        uri = self.reverse('book')
        response = self.client.get(uri)
        self.assertContains(
            response, 'count="%s"' % Book.objects.all().count())
        book = Book.objects.create(author=self.author, title="book", status=1)
        response = self.client.get(uri)
        self.assertContains(
            response, 'count="%s"' % Book.objects.all().count())

        uri = self.reverse('author-test-book-article',
                           book=book.pk) + "?author=" + str(self.author.pk)
        response = self.client.delete(
            uri,
            HTTP_AUTHORIZATION=self.author.user.accesskey_set.get().key)
        self.assertContains(response, 'Some error', status_code=500)
        # self.assertEqual(len(mail.outbox), 1)

        self.assertEqual(
            mail.outbox[
                -1].subject, '[Django] ADREST API Error (500): /1.0.0/owner/test/book/%s/article/' % Book.objects.all().count())  # noqa

        response = self.client.put(
            uri, HTTP_AUTHORIZATION=self.author.user.accesskey_set.get().key)
        self.assertContains(response, 'Assertion error', status_code=400)
        # self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(
            mail.outbox[
                -1].subject, '[Django] ADREST API Error (400): /1.0.0/owner/test/book/%s/article/' % Book.objects.all().count())  # noqa

        response = self.post_resource('book')
        self.assertContains(response, '{"error": "\'Frozen', status_code=400)

    def test_some_other(self):
        response = self.get_resource('test')
        self.assertContains(response, 'count="3"')
        self.assertContains(response, '<someother>1</someother>')
        self.assertContains(response, '<method>GET</method>')

    def test_books(self):
        """Test response Link header

        Example: </1.0.0/author/5/test/book/?page=2>; rel="next"
        """
        link_re = re.compile(r'<(?P<link>[^>]+)>\; rel=\"(?P<rel>[^\"]+)\"')

        response = self.get_resource('author-test-book',
                                     data=dict(author=self.author.pk))
        self.assertTrue(response.has_header("Link"))

        # Get objects by links on Link header
        response = self.client.get(link_re.findall(response['Link'])[0][0])

        links = link_re.findall(response['Link'])

        self.assertEquals(links[0][1], 'next')
        self.assertEquals(links[1][1], 'previous')

        response = self.get_resource(
            'author-test-book', data={
                'author': self.author.pk, 'adr-max': 0
            })
        self.assertFalse(response.has_header("Link"))

        response = self.get_resource(
            'author-test-book',
            data={
                'author': self.author.pk, 'adr-max': 'all'
            })
        self.assertEquals(response.status_code, 200)
        self.assertFalse(response.has_header("Link"))


class AdrestMapTest(TestCase):

    def setUp(self):
        self.client = Client()

    def test_methods(self):
        uri = reverse("main-%s-map" % str(api))
        self.assertEqual(uri, "/%s/map" % api)
        response = self.client.get(uri)
        self.assertContains(response, 'API')
        self.assertContains(response, 'nickname')

        response = self.client.get(uri, HTTP_ACCEPT="application/json")
        self.assertContains(response, '"price"')


# lint_ignore=F0401,C0110



from django.test import TestCase
from mixer.backend.django import mixer


class CoreSerializerTest(TestCase):

    def setUp(self):
        for _ in range(1, 10):
            mixer.blend('main.book')

    def test_base_types(self):
        """ Testing serialization of base types.
        """
        from adrest.utils.serializer import BaseSerializer
        try:
            from collections import OrderedDict
        except ImportError:
            from ordereddict import OrderedDict # noqa

        from datetime import datetime
        from decimal import Decimal

        serializer = BaseSerializer()
        data = dict(
            string_='test',
            unicode_=unicode('test'),
            datetime_=datetime(2007, 01, 01),
            odict_=OrderedDict(value=1),
            dict_=dict(
                list_=[1, 2.35, Decimal(3), False]
            )
        )

        value = serializer.serialize(data)
        self.assertEqual(value, dict(
            string_=u'test',
            unicode_=u'test',
            datetime_='2007-01-01T00:00:00',
            odict_=dict(value=1),
            dict_=dict(
                list_=[1, 2.35, 3.0, False]
            )
        ))

    def test_django_model(self):
        from adrest.utils.serializer import BaseSerializer

        pirate = mixer.blend('core.pirate', name='Billy')
        data = [
            mixer.blend('core.boat', pirate=pirate),
            mixer.blend('core.boat', pirate=pirate),
            28, 'string']

        serializer = BaseSerializer(
            exclude='fake',
            include='pk',
            related=dict(
                pirate=dict(fields='character')
            ),
        )
        self.assertEqual(serializer.model_options['exclude'], set(['fake']))

        out = serializer.to_simple(data, **serializer.model_options)
        self.assertTrue(out[0]['fields']['pk'])
        self.assertEqual(out[0]['fields']['pirate']['fields']['character'],
                         data[0].pirate.character)

        # Test m2o serialization
        serializer = BaseSerializer(
            include="boat_set",
            related=dict(
                boat_set=dict(
                    fields=[])
            ),
        )
        out = serializer.to_simple(pirate, **serializer.model_options)

        self.assertEquals(len(out['fields']['boat_set']), 2)
        for boat in out['fields']['boat_set']:
            self.assertEquals(boat['fields']['pirate'], pirate.pk)
            self.assertTrue('title' in boat['fields'].keys())

        out = serializer.to_simple(pirate)
        self.assertTrue('model' in out)

        out = serializer.to_simple(pirate, include=['boat_set'])
        self.assertTrue(out['fields']['boat_set'])
        self.assertEqual(len(list(out['fields']['boat_set'])), 2)

    def test_paginator(self):
        from adrest.mixin import EmitterMixin
        from django.views.generic import View
        from django.test import RequestFactory
        from tests.core.models import Pirate
        from adrest.utils.paginator import Paginator

        mixer.cycle(3).blend('core.pirate')

        class SomeResource(EmitterMixin, View):

            class Meta:
                model = 'core.pirate'
                dyn_prefix = 'adr-'
                limit_per_page = 2

            def dispatch(self, request, **resources):
                p = Paginator(request, self, Pirate.objects.all())
                return self.emit(p, request=request)

        rf = RequestFactory()
        resource = SomeResource()

        response = resource.dispatch(rf.get('/'))
        self.assertContains(response, '"page": 1')
        self.assertContains(response, '"num_pages": 2')

    def test_xml(self):
        from adrest.utils.serializer import XMLSerializer
        from ...main.models import Book

        for _ in range(1, 10):
            mixer.blend(Book)
        worker = XMLSerializer()
        test = worker.serialize(Book.objects.all())
        self.assertTrue("author" in test)

    def test_json(self):
        from ...main.models import Author
        from adrest.utils.serializer import JSONSerializer

        authors = Author.objects.all()
        worker = JSONSerializer(options=dict(
            separators=(',', ':')
        ))
        test = worker.serialize(authors)
        self.assertTrue("main.author" in test)
        self.assertTrue('":{"' in test)



from StringIO import StringIO

from django import http
from django.core.serializers.json import Serializer
from django.db.models.query import QuerySet
from django.views.generic import TemplateView


class TemplateContextView(TemplateView):
    """ Allow define context in as_view method.
    """
    context = dict()

    def __init__(self, context=None, **kwargs):
        self.context = context or dict()
        super(TemplateContextView, self).__init__(**kwargs)

    def get(self, request, *args, **kwargs):
        self.context.update(self.get_context_data(**kwargs))
        return self.render_to_response(self.context)


class AbstractResponseMixin(object):
    """ Abstract class for data serialize.
    """
    mimetype = "application/text"

    @staticmethod
    def render_template(context):
        "String representation of given context."
        return str(context)

    def render_to_response(self, context):
        "Return HttpResponse."
        return http.HttpResponse(
            self.render_template(context),
            content_type=self.mimetype)


class JSONResponseMixin(AbstractResponseMixin):
    """ Serialize queryset or any objects context in JSON.
    """
    mimetype = "application/json"

    def render_template(self, context):
        encoder = Serializer()
        if isinstance(context, QuerySet):
            return encoder.serialize(context, ensure_ascii=False)

        else:
            encoder.objects = context
            encoder.options = dict()
            encoder.stream = StringIO()
            encoder.end_serialization()
            return encoder.getvalue()


class JSONView(JSONResponseMixin, TemplateView):
    """ Render view context in JSON.
    """
    pass



from hashlib import md5
from os import path
from time import time


def upload_to(instance, filename, prefix=None):
    """ Auto upload function for File and Image fields.
    """
    ext = path.splitext(filename)[1]
    name = str(instance.pk or time()) + filename

    # We think that we use utf8 based OS file system
    filename = md5(name.encode('utf8')).hexdigest() + ext
    basedir = path.join(instance._meta.app_label, instance._meta.module_name)
    if prefix:
        basedir = path.join(basedir, prefix)
    return path.join(basedir, filename[:2], filename[2:4], filename)



""" Authentication support. """

import abc
import base64

from django.contrib.auth import authenticate


__all__ = ('AnonimousAuthenticator', 'BasicAuthenticator', 'UserAuthenticator',
           'UserLoggedInAuthenticator', 'AbstractAuthenticator')


class AbstractAuthenticator(object):

    """ Abstract Authenticator. """

    __meta__ = abc.ABCMeta

    def __init__(self, resource):
        self.resource = resource
        self.identifier = ''

    @abc.abstractmethod
    def authenticate(self, request):
        raise NotImplementedError

    @abc.abstractmethod
    def configure(self, request):
        raise NotImplementedError

    @staticmethod
    def test_rights(resources, request=None):
        return True

    @staticmethod
    def get_fields():
        return []


class AnonimousAuthenticator(AbstractAuthenticator):

    """ Anonymous access. Set identifier by IP address. """

    def authenticate(self, request):
        return True

    def configure(self, request):
        pass


class UserLoggedInAuthenticator(AbstractAuthenticator):

    """ Check current session for user is authenticated. """

    def __init__(self, *args, **kwargs):
        self.user = None
        super(UserLoggedInAuthenticator, self).__init__(*args, **kwargs)

    def authenticate(self, request):
        user = getattr(request, 'user', None)
        return user and user.is_active

    def configure(self, request):
        self.user = request.user
        self.resource.identifier = self.user.username


class BasicAuthenticator(UserLoggedInAuthenticator):

    """ HTTP Basic Authentication. """

    def authenticate(self, request=None):
        if 'HTTP_AUTHORIZATION' in request.META:
            auth = request.META['HTTP_AUTHORIZATION'].split()
            if len(auth) == 2 and auth[0].lower() == "basic":
                uname, passwd = base64.b64decode(auth[1]).split(':')
                user = authenticate(username=uname, password=passwd)
                if user and user.is_active:
                    request.user = user
                    return True

        return False


class UserAuthenticator(UserLoggedInAuthenticator):

    """ Authorization by login and password. """

    username_fieldname = 'username'
    password_fieldname = 'password'

    def authenticate(self, request=None):
        try:
            username = request.REQUEST.get(self.username_fieldname)
            password = request.REQUEST.get(self.password_fieldname)
            request.user = authenticate(username=username, password=password)
            return request.user and request.user.is_active

        except KeyError:
            return False

    @classmethod
    def get_fields(cls):
        return [
            (cls.username_fieldname, dict(required=True)),
            (cls.password_fieldname, dict(required=True))]


try:
    from ..models import AccessKey

    class AccessKeyAuthenticator(UserLoggedInAuthenticator):

        """ Authorization by API token. """

        def authenticate(self, request=None):
            """ Authenticate using AccessKey from HTTP headers or GET params. """
            try:
                token = request.META.get('HTTP_AUTHORIZATION') or request.REQUEST['key']
                accesskey = AccessKey.objects.select_related('user').get(key=token)
                request.user = accesskey.user
                return request.user and request.user.is_active

            except(KeyError, AccessKey.DoesNotExist):
                return False

    __all__ += 'AccessKeyAuthenticator',

except ImportError:
    pass



from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.views.defaults import page_not_found, server_error

from . import views as mainviews
from api.api import api

try:
    from django.conf.urls import url, patterns, include
# Support Django<=1.5
except ImportError:
    from django.conf.urls.defaults import url, patterns, include


# 404 and 500 handlers
handler404 = page_not_found
handler500 = server_error

# Project urls
urlpatterns = patterns(
    '',
    url('^$', mainviews.Index.as_view(), name='index'),
    url('^logout/$', 'django.contrib.auth.views.logout', name='logout'),

    # Api urls
    url('^api/', include(api.urls)),
)

# Django admin
admin.autodiscover()
urlpatterns += [url(r'^admin/', include(admin.site.urls)), ]

# Debug static files serve
urlpatterns += staticfiles_urlpatterns()

# lint_ignore=E1120



from .models import Book

from django.http import HttpResponse

from adrest.utils.emitter import JSONEmitter
from adrest.utils.exceptions import HttpError
from adrest.views import ResourceView


class AuthorResource(ResourceView):

    class Meta:
        allowed_methods = 'GET', 'POST', 'PATCH'
        model = 'main.author'
        url_regex = '^owner/$'


class BookResource(ResourceView):

    class Meta:
        allowed_methods = 'GET', 'post', 'pUt', 'DELETE'
        parent = AuthorResource
        model = 'main.book'


class BookPrefixResource(BookResource):

    class Meta:
        prefix = 'test'


class ArticleResource(ResourceView):

    class Meta:
        allowed_methods = 'GET', 'PUT', 'DELETE'
        parent = BookPrefixResource
        model = 'main.article'

    def put(self, request, **kwargs):
        assert False, "Assertion error"

    def delete(self, request, **kwargs):
        raise Exception("Some error")


class OtherResource(ResourceView):

    class Meta:
        parent = BookResource

    def get(self, request, **kwargs):
        return True


class SomeOtherResource(ResourceView):

    class Meta:
        parent = AuthorResource
        url_params = 'device',

    def get(self, request, **kwargs):
        return self.paginate(request, [1, 2, 3])


class CustomResource(ResourceView):

    class Meta:
        allowed_methods = 'GET', 'POST'
        model = 'main.book'
        queryset = Book.objects.all()
        emit_template = 'main/custom.xml'

    def get(self, request, **kwargs):
        return list(self._meta.queryset)

    def post(self, request, **resources):
        try:
            request.data['test'] = 123
        except TypeError, e:
            raise HttpError(dict(error=str(
                e)), status=400, emitter=JSONEmitter)


class DummyResource(ResourceView):

    class Meta:
        name = 'iamdummy'

    def get(self, request, **resources):
        return True


class CSVResource(ResourceView):

    class Meta:
        allowed_methods = 'GET'

    def get(self, request, **resources):
        return HttpResponse('value'.encode("utf-16"), content_type="text/csv")

# lint_ignore=C



""" **ADREST** is an API framework for Django.

It supports REST_ and RPC_ paradigms.

:copyright: 2013 by Kirill Klenov.
:license: BSD, see LICENSE for more details.

"""

__version__ = "3.3.1"
__project__ = "adrest"
__author__ = "Kirill Klenov <horneds@gmail.com>"
__license__ = "GNU LGPL"

version_info = [p for p in map(int, __version__.split("."))]

try:
    from django.conf import settings as django_settings
    if django_settings.configured:

        if 'adrest' not in django_settings.INSTALLED_APPS:
            import logging
            logging.warn('You should added "adrest" to INSTALLED_APPS.')

except ImportError:
    pass

from .api import Api
from .views import ResourceView
from .utils.auth import *
from .utils.emitter import *
from .utils.parser import *
from .utils.serializer import *
from .utils.throttle import *
from .utils.exceptions import HttpError

# pylama:ignore=W0401,W0611



"""
You can use :class:`adrest.Api` for bind multiple :class:`adrest.ResourceView`
together with version prefix.

Create API
----------

Default use: ::

    from adrest.api import Api
    from myresources import Resource1, Resource2

    api = Api('0.1')


Register a resource
-------------------
After creation you can register some resources with that Api. ::

    api.register(Resource1)
    api.register(Resource2)

You can use `api.register` method as decorator: ::

    @api.register
    class SomeResource():
        ...


Enable API Map
--------------

You can enable API Map for quick reference on created resources. Use `api_map`
param. ::

    api = Api('1.0b', api_map=True)

By default access to map is anonimous. If you want use a custom authenticator
register map resource by manualy. ::

    from adrest.resources.map import MapResource

    api = Api('1.0')
    api.register(MapResource, authenticators=UserLoggedInAuthenticator)


Auto JSONRPC from REST
----------------------

If you are created some REST api with adrest, you already have JSON RPC too.
Use `api_rpc` param. ::

    api = Api('1.0', api_rpc=True)


"""
import logging

from django.conf.urls import patterns
from django.dispatch import Signal
from django.http import HttpRequest

from .resources.map import MapResource
from .resources.rpc import AutoJSONRPC
from .views import ResourceView
from .utils import exceptions, status, emitter


__all__ = 'Api',


logger = logging.getLogger('adrest')


class Api(object):

    """ Implements a registry to tie together resources that make up an API.

    Especially useful for navigation, providing multiple versions of your
    API.

    :param version: Version info as string or iterable.
    :param api_map: Enable :ref:`apimap`
    :param api_prefix: Prefix for URL and URL-name
    :param api_rpc: Enable :ref:`jsonrpc`
    :param **meta: Redefine Meta options for resource classes

    """

    def __init__(self, version=None, api_map=True, api_prefix='api',
                 api_rpc=False, **meta):
        self.version = self.str_version = version
        self.prefix = api_prefix
        self.resources = dict()
        self.request_started = Signal()
        self.request_finished = Signal()

        if not isinstance(self.str_version, basestring):
            try:
                self.str_version = '.'.join(map(str, version or list()))
            except TypeError:
                self.str_version = str(version)

        self.meta = dict()

        if api_map:
            self.register(MapResource)

        if api_rpc:
            self.register(AutoJSONRPC, emitters=[
                emitter.JSONPEmitter, emitter.JSONEmitter
            ])

        self.meta = meta

    def __str__(self):
        return self.str_version

    def register(self, resource=None, **meta):
        """ Add resource to the API.

        :param resource: Resource class for registration
        :param **meta: Redefine Meta options for the resource

        :return adrest.views.Resource: Generated resource.

        """
        if resource is None:
            def wrapper(resource):
                return self.register(resource, **meta)
            return wrapper

        # Must be instance of ResourceView
        if not issubclass(resource, ResourceView):
            raise AssertionError("%s not subclass of ResourceView" % resource)

        # Cannot be abstract
        if resource._meta.abstract:
            raise AssertionError("Attempt register of abstract resource: %s."
                                 % resource)

        # Fabric of resources
        meta = dict(self.meta, **meta)
        meta['name'] = meta.get('name', resource._meta.name)
        options = type('Meta', tuple(), meta)

        params = dict(api=self, Meta=options, **meta)

        params['__module__'] = '%s.%s' % (
            self.prefix, self.str_version.replace('.', '_'))

        params['__doc__'] = resource.__doc__

        new_resource = type(
            '%s%s' % (resource.__name__, len(self.resources)),
            (resource,), params)

        if self.resources.get(new_resource._meta.url_name):
            logger.warning(
                "A resource '%r' is replacing the existing record for '%s'",
                new_resource, self.resources.get(new_resource._meta.url_name))

        self.resources[new_resource._meta.url_name] = new_resource

        return resource

    @property
    def urls(self):
        """ Provide URLconf details for the ``Api``.

        And all registered ``Resources`` beneath it.

            :return list: URL's patterns

        """
        urls = []

        for url_name in sorted(self.resources.keys()):

            resource = self.resources[url_name]
            urls.append(resource.as_url(
                api=self,
                name_prefix='-'.join(
                    (self.prefix, self.str_version)).strip('-'),
                url_prefix=self.str_version
            ))

        return patterns(self.prefix, *urls)

    def call(self, name, request=None, **params):
        """ Call resource by ``Api`` name.

        :param name: The resource's name (short form)
        :param request: django.http.Request instance
        :param **params: Params for a resource's call

        :return object: Result of resource's execution

        """
        if name not in self.resources:
            raise exceptions.HttpError(
                'Unknown method \'%s\'' % name,
                status=status.HTTP_501_NOT_IMPLEMENTED)
        request = request or HttpRequest()
        resource = self.resources[name]
        view = resource.as_view(api=self)
        return view(request, **params)

    @property
    def testCase(self):
        """ Generate class for testing this API.

        :return TestCase: A testing class

        """
        from adrest.tests import AdrestTestCase

        return type('TestCase', (AdrestTestCase, ), dict(api=self))



import collections


def as_tuple(obj):
    " Given obj return a tuple "

    if not obj:
        return tuple()

    if isinstance(obj, (tuple, set, list)):
        return tuple(obj)

    if hasattr(obj, '__iter__') and not isinstance(obj, dict):
        return obj

    return obj,


def gen_url_name(resource):
    " URL name for resource class generator. "

    if resource._meta.parent:
        yield resource._meta.parent._meta.url_name

    if resource._meta.prefix:
        yield resource._meta.prefix

    for p in resource._meta.url_params:
        yield p

    yield resource._meta.name


def gen_url_regex(resource):
    " URL regex for resource class generator. "

    if resource._meta.parent:
        yield resource._meta.parent._meta.url_regex.rstrip('/$').lstrip('^')

    for p in resource._meta.url_params:
        yield '%(name)s/(?P<%(name)s>[^/]+)' % dict(name=p)

    if resource._meta.prefix:
        yield resource._meta.prefix

    yield '%(name)s/(?P<%(name)s>[^/]+)?' % dict(name=resource._meta.name)


def fix_request(request):
    methods = "PUT", "PATCH"

    if request.method in methods and not getattr(request, request.method, None):

        if hasattr(request, '_post'):
            del(request._post)
            del(request._files)

        if hasattr(request, '_request'):
            del(request._request)

        request.method, method = "POST", request.method
        setattr(request, method, request.POST)
        request.method = method

    request.adrest_fixed = True

    return request


class FrozenDict(collections.Mapping): # nolint
    """ Immutable dict. """

    def __init__(self, *args, **kwargs):
        self._d = dict(*args, **kwargs)
        self._hash = None

    def __iter__(self):
        return iter(self._d)

    def __len__(self):
        return len(self._d)

    def __getitem__(self, key):
        return self._d[key]

    def __hash__(self):
        # It would have been simpler and maybe more obvious to
        # use hash(tuple(sorted(self._d.iteritems()))) from this discussion
        # so far, but this solution is O(n). I don't know what kind of
        # n we are going to run into, but sometimes it's hard to resist the
        # urge to optimize when it will gain improved algorithmic performance.
        if self._hash is None:
            self._hash = 0
            for key, value in self.iteritems():
                self._hash ^= hash(key)
                self._hash ^= hash(value)
        return self._hash

    def __str__(self):
        return str(dict(self.iteritems()))

    def __repr__(self):
        return "<FrozenDict: %s>" % repr(dict(self.iteritems()))



""" Test ADRest API module. """
from django.test import TestCase, RequestFactory

from adrest.views import ResourceView
from adrest.utils.emitter import XMLEmitter


class CoreApiTest(TestCase):

    """ Test api. """

    def test_base(self):
        """ Test main functionality. """
        from adrest.api import Api

        api = Api('1.0.0')
        self.assertEqual(api.version, '1.0.0')
        self.assertTrue(api.urls)

        class Resource(ResourceView):

            class Meta:
                model = 'core.pirate'

        api.register(Resource)
        self.assertEqual(len(api.urls), 2)
        self.assertTrue(api.resources.get('pirate'))

        class TreasureResource(ResourceView):

            class Meta:
                parent = Resource
                model = 'core.treasure'

        api.register(TreasureResource)
        self.assertEqual(len(api.urls), 3)
        self.assertTrue(api.resources.get('pirate-treasure'))

        class PrefixResource(TreasureResource):
            class Meta:
                prefix = 'more'

        api.register(PrefixResource)
        self.assertEqual(len(api.urls), 4)
        self.assertTrue(api.resources.get('pirate-more-treasure'))

        class Resource2(ResourceView):

            class Meta:
                model = 'core.pirate'
                emitters = XMLEmitter

        api.register(Resource2, name='wow')
        resource = api.resources.get('wow')
        self.assertEqual(resource._meta.emitters, (XMLEmitter,))
        self.assertEqual(resource._meta.name, 'wow')

    def test_register(self):
        """ Test register method. """
        from adrest.api import Api

        api = Api('1.0.0')

        class TestResource(ResourceView):

            class Meta:
                name = 'test1'

        api.register(TestResource, name='test2')
        resource = api.resources.get('test2')
        self.assertEqual(resource._meta.name, 'test2')

        @api.register
        class TestResource(ResourceView):

            class Meta:
                name = 'test3'

        self.assertTrue('test3' in api.resources)

        @api.register()
        class TestResource(ResourceView):

            class Meta:
                name = 'test4'

        self.assertTrue('test4' in api.resources)

        @api.register(name='test6')
        class TestResource(ResourceView):

            class Meta:
                name = 'test5'

        self.assertFalse('test5' in api.resources)
        self.assertTrue('test6' in api.resources)

    def test_fabric(self):
        from adrest.api import Api

        api = Api('1.0.0')

        @api.register
        class PirateResource(ResourceView):

            class Meta:
                model = 'core.pirate'

            def get_collection(self, request, **resources):
                return super(PirateResource, self).get_collection(
                    request, **resources)

        resource = api.resources['pirate']
        rf = RequestFactory()
        response = resource().dispatch(rf.get('/'))
        self.assertContains(response, 'resources')

    def test_version(self):
        """ Test version. """
        from ..api import api2

        resource = api2.resources.get('pirate')
        self.assertEqual(resource.api, api2)

        uri = api2.testCase.reverse('pirate')
        self.assertEqual(uri, '/pirates2/1.0.0/pirate/')



# lint_ignore=E0102,W0404,C0110



from django.test import TestCase
from adrest.utils import tools
from adrest.tests import AdrestRequestFactory


class UtilsTests(TestCase):

    def test_as_tuple(self):
        self.assertEqual(tools.as_tuple(None), tuple())
        self.assertEqual(tools.as_tuple(''), tuple())
        self.assertEqual(tools.as_tuple([]), tuple())
        self.assertEqual(tools.as_tuple([1, 2]), (1, 2))
        self.assertEqual(tools.as_tuple(set([1, 2])), (1, 2))
        self.assertEqual(tools.as_tuple({1: 1}), ({1: 1},))
        test = object()
        self.assertEqual(tools.as_tuple(test), (test,))

    def test_fix_request(self):
        rf = AdrestRequestFactory()
        request = rf.put('/test', {
            'foo': 'bar'
        })
        self.assertFalse(request.REQUEST.items())

        fixed = tools.fix_request(request)
        self.assertTrue(fixed.adrest_fixed)
        self.assertTrue(fixed.REQUEST.items())



""" Safe API. """
from ..utils import status
from ..utils.meta import MixinBaseMeta
from ..utils.exceptions import HttpError
from ..utils.throttle import NullThrottle, AbstractThrottle

__all__ = 'ThrottleMixin',


class ThrottleMeta(MixinBaseMeta):

    """ Prepare throtles. """

    def __new__(mcs, name, bases, params):
        cls = super(ThrottleMeta, mcs).__new__(mcs, name, bases, params)

        if not issubclass(cls._meta.throttle, AbstractThrottle):
            raise AssertionError(
                "'cls.Meta.throttle' must be subclass of AbstractThrottle"
            )

        return cls


class ThrottleMixin(object):

    """ Throttle request. """

    __metaclass__ = ThrottleMeta

    class Meta:
        throttle = NullThrottle

    def throttle_check(self):
        """ Check for throttling. """
        throttle = self._meta.throttle()
        wait = throttle.should_be_throttled(self)
        if wait:
            raise HttpError(
                "Throttled, wait {0} seconds.".format(wait),
                status=status.HTTP_503_SERVICE_UNAVAILABLE)



import operator

from django.db.models import signals
from django.db.models.expressions import F, ExpressionNode


EXPRESSION_NODE_CALLBACKS = {
    ExpressionNode.ADD: operator.add,
    ExpressionNode.SUB: operator.sub,
    ExpressionNode.MUL: operator.mul,
    ExpressionNode.DIV: operator.div,
    ExpressionNode.MOD: operator.mod,
    ExpressionNode.AND: operator.and_,
    ExpressionNode.OR: operator.or_,
}


class CannotResolve(Exception):
    pass


def _resolve(instance, node):
    if isinstance(node, F):
        return getattr(instance, node.name)
    elif isinstance(node, ExpressionNode):
        return _resolve(instance, node)
    return node


def resolve_expression_node(instance, node):
    op = EXPRESSION_NODE_CALLBACKS.get(node.connector, None)
    if not op:
        raise CannotResolve
    runner = _resolve(instance, node.children[0])
    for n in node.children[1:]:
        runner = op(runner, _resolve(instance, n))
    return runner


def update(instance, full_clean=True, post_save=False, **kwargs):
    "Atomically update instance, setting field/value pairs from kwargs"

    # apply the updated args to the instance to mimic the change
    # note that these might slightly differ from the true database values
    # as the DB could have been updated by another thread. callers should
    # retrieve a new copy of the object if up-to-date values are required
    for k, v in kwargs.iteritems():
        if isinstance(v, ExpressionNode):
            v = resolve_expression_node(instance, v)
        setattr(instance, k, v)

    # clean instance before update
    if full_clean:
        instance.full_clean()

    # fields that use auto_now=True should be updated corrected, too!
    for field in instance._meta.fields:
        if hasattr(field, 'auto_now') and field.auto_now and field.name not in kwargs:
            kwargs[field.name] = field.pre_save(instance, False)

    rows_affected = instance.__class__._default_manager.filter(
        pk=instance.pk).update(**kwargs)

    if post_save:
        signals.post_save.send(sender=instance.__class__, instance=instance)

    return rows_affected


class Choices(object):

    def __init__(self, *choices):
        self._choices = []
        self._choice_dict = {}
        self._labels = {}

        for choice in choices:
            if isinstance(choice, (list, tuple)):
                if len(choice) == 2:
                    choice = (choice[0], choice[0], choice[1])

                elif len(choice) != 3:
                    raise ValueError("Choices can't handle a list/tuple of length %s, only 2 or 3" % len(choice))
            else:
                choice = (choice, choice, choice)

            self._choices.append((choice[0], choice[2]))
            self._choice_dict[choice[1]] = choice[0]

    def __getattr__(self, attname):
        try:
            return self._choice_dict[attname]
        except KeyError:
            raise AttributeError(attname)

    def __iter__(self):
        return iter(self._choices)

    def __getitem__(self, index):
        return self._choices[index]

    def __repr__(self):
        values, names = zip(*self._choices)
        labels = self._labels.itervalues()
        return '%s(%s)' % (self.__class__.__name__,
                           repr(zip(values, labels, names)))



from .resources import (
    AuthorResource, BookPrefixResource, ArticleResource, SomeOtherResource,
    CustomResource, CSVResource)
from adrest.api import Api
from adrest.utils.auth import AnonimousAuthenticator, AccessKeyAuthenticator, \
    UserAuthenticator
from adrest.utils.emitter import XMLTemplateEmitter, JSONEmitter
from adrest.utils.throttle import CacheThrottle


class CustomUserAuth(UserAuthenticator):
    username_fieldname = 'nickname'


API = Api(
    version=(1, 0, 0), emitters=(XMLTemplateEmitter, JSONEmitter),
    throttle=CacheThrottle, api_prefix='main')

API.register(AuthorResource,
             authenticators=(CustomUserAuth, AnonimousAuthenticator))
API.register(BookPrefixResource)
API.register(CustomResource)
API.register(ArticleResource, authenticators=AccessKeyAuthenticator)
API.register(SomeOtherResource, url_name='test', url_regex='test/mem/$')
API.register(CSVResource)

# lint_ignore=C



""" Setup ADRest. """
import re
import sys
from os import path as op, walk

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


_read = lambda f: open(
    op.join(op.dirname(__file__), f)).read() if op.exists(f) else ''

_meta = _read('adrest/__init__.py')
_license = re.search(r'^__license__\s*=\s*"(.*)"', _meta, re.M).group(1)
_project = re.search(r'^__project__\s*=\s*"(.*)"', _meta, re.M).group(1)
_version = re.search(r'^__version__\s*=\s*"(.*)"', _meta, re.M).group(1)

install_requires = [
    l for l in _read('requirements.txt').split('\n')
    if l and not l.startswith('#')]

tests_require = [
    l for l in _read('requirements-tests.txt').split('\n')
    if l and not l.startswith('#')]

package_data = []

for folder in ['templates']:
    for root, dirs, files in walk(op.join(_project, folder)):
        for filename in files:
            package_data.append("%s/%s" % (root[len(_project) + 1:], filename))


class __PyTest(TestCommand):

    test_args = []
    test_suite = True

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


setup(
    author_email='horneds@gmail.com',
    author='Kirill Klenov',
    description=_read('DESCRIPTION'),
    install_requires=install_requires,
    license=_license,
    long_description=_read('README.rst'),
    name=_project,
    package_data={'': package_data},
    packages=find_packages(),
    platforms=('Any'),
    keywords='rest rpc api django'.split(),
    tests_require=tests_require,
    cmdclass={'test': __PyTest},
    url=' http://github.com/klen/{0}'.format(_project),
    version=_version,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)', # noqa
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities'
    ],
)

# pylama:ignore=E731



""" Default ADRest form for Django models. """
from django.db.models import Model
from django.db.models.fields import AutoField
from django.db.models.fields.related import ManyToManyField
from django.forms.models import ModelForm


class PartitialForm(ModelForm):

    """ Default ADRest form for models.

    Allows partitial updates and parses a finded resources.

    """

    def __init__(self, data=None, instance=None, initial=None, prefix=None,
                 label_suffix=':', empty_permitted=False, **kwargs):

        formdata = self.__get_initial_from_model(instance)
        formdata.update(data or dict())

        resources = dict((k, v if not isinstance(
            v, Model) else v.pk) for k, v in kwargs.iteritems())
        formdata.update(resources)

        super(PartitialForm, self).__init__(
            formdata, instance=instance, prefix=prefix,
            label_suffix=label_suffix, empty_permitted=empty_permitted)

    def __get_initial_from_model(self, instance):
        required_fields = [
            f for f in self._meta.model._meta.fields
            if not isinstance(f, (ManyToManyField, AutoField))
        ]

        gen = (
            (f.name, f.value_from_object(instance) if instance
             else f.get_default()) for f in required_fields
        )
        return dict(item for item in gen if not item[1] is None)



from django.http import HttpResponse

from .status import HTTP_200_OK


class SerializedMeta(type):

    def __call__(cls, content, *args, **kwargs):
        """ Don't create clones.
        """
        if isinstance(content, HttpResponse):
            return content

        return super(SerializedMeta, cls).__call__(
            content, *args, **kwargs
        )


class SerializedHttpResponse(HttpResponse): # nolint
    """ Response has will be serialized.
        Django http response will be returned as is.

        :param error: Force error in response.
    """
    __metaclass__ = SerializedMeta

    def __init__(self, content='', status=None,
                 content_type=None, error=False):
        """
            Save original response.
        """
        self.response = content
        self._error = error
        self._content_type = content_type

        super(SerializedHttpResponse, self).__init__(
            content,
            status=status,
            content_type=content_type)

    @property
    def error(self):
        return self._error or self.status_code != HTTP_200_OK

    def __repr__(self):
        return "<SerializedHttpResponse %s>" % self.status_code

# pymode:lint_ignore=E1103



""" Tests ADRest auth mixin.
"""
from django.views.generic import View

from ..api import api as API
from adrest.mixin import AuthMixin
from adrest.tests import AdrestTestCase
from adrest.utils.auth import UserAuthenticator


class CoreAuthTest(AdrestTestCase):

    """ Auth related tests. """

    api = API

    def test_meta(self):
        """ Test a meta attribute generation. """

        class Resource(View, AuthMixin):

            class Meta:
                model = 'core.pirate'
                authenticators = UserAuthenticator

        self.assertTrue(Resource._meta)
        self.assertTrue(Resource._meta.authenticators)
        self.assertEqual(Resource._meta.authenticators, (UserAuthenticator,))

# lint_ignore=W0212



""" Production's settings.
"""
from .core import * # nolint

# Hack for adrest loading in example project
import sys
sys.path.insert(0, op.abspath(op.dirname(op.dirname(op.dirname(__file__)))))


ENVIRONMENT_NAME = 'production'

# Applications
INSTALLED_APPS += (

    # Community apps
    'south',

    # Base project app
    'main',

    # API
    'adrest',

)

# Caches
CACHES['default']['KEY_PREFIX'] = '_'.join((PROJECT_NAME, 'PRJ'))

# Sessions
SESSION_ENGINE = "django.contrib.sessions.backends.cache"

# Templates cache
TEMPLATE_LOADERS = (
    ('django.template.loaders.cached.Loader', TEMPLATE_LOADERS),
)

# Logging
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
        },
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
        'celery': {
            'handlers': ['console'],
            'level': 'ERROR',
        },
    }
}

logging.info('Production settings are loaded.')



""" Add ADREST related models to Django admin. """

from django.contrib import admin


try:
    from .models import Access

    class AccessAdmin(admin.ModelAdmin):

        """ Support access log in django admin. """

        list_display = (
            'created_at',
            'identifier',
            'method',
            'status_code',
            'uri',
            'version'
        )
        list_filter = 'method', 'version'
        search_fields = 'uri', 'identifier'
        date_hierarchy = 'created_at'

    admin.site.register(Access, AccessAdmin)

except ImportError:
    pass


try:
    from .models import AccessKey

    class AccessKeyAdmin(admin.ModelAdmin):

        """ Support access keys in django admin. """

        list_display = 'key', 'user', 'created'
        search_fields = '=key', '=user'
        raw_id_fields = 'user',

    admin.site.register(AccessKey, AccessKeyAdmin)

except ImportError:
    pass



""" Models for tests. """

from django.db import models


class Pirate(models.Model):

    """ Mighty pirates. """

    name = models.CharField(max_length=50)
    captain = models.BooleanField(default=False)
    character = models.CharField(max_length=10, choices=(
        ('good', 'good'),
        ('evil', 'evil'),
        ('sorrow', 'sorrow'),
    ))

    def __unicode__(self):
        return self.name


class Island(models.Model):

    """ Magical islands. """

    title = models.CharField(max_length=50)


class Treasure(models.Model):

    """ Incrediable treasures. """

    created_at = models.DateTimeField(auto_now_add=True)
    pirate = models.ForeignKey(Pirate, null=True, blank=True)
    island = models.ForeignKey(Island)


class Boat(models.Model):

    """ Fastest boats. """

    title = models.CharField(max_length=50)
    pirate = models.ForeignKey(Pirate)



from mixer.backend.django import mixer

from ..api import api as API
from adrest.tests import AdrestTestCase
from adrest.views import ResourceView


class CoreResourceTest(AdrestTestCase):

    api = API

    def test_meta(self):

        class AlphaResource(ResourceView):
            pass
        self.assertEqual(AlphaResource._meta.name, 'alpha')

        class BetaResource(ResourceView):
            pass
        self.assertEqual(BetaResource._meta.name, 'beta')
        self.assertEqual(BetaResource._meta.url_name, 'beta')

        class GammaResource(ResourceView):

            class Meta:
                parent = BetaResource

        self.assertEqual(GammaResource._meta.parents, [BetaResource])
        self.assertEqual(GammaResource._meta.name, 'gamma')
        self.assertEqual(GammaResource._meta.url_name, 'beta-gamma')

    def test_resources(self):

        pirate = mixer.blend('core.pirate')
        for _ in xrange(3):
            mixer.blend('core.boat', pirate=pirate)

        self.assertEqual(pirate.boat_set.count(), 3)

        self.delete_resource('pirate-boat', pirate=pirate, data=dict(
            boat=[b.pk for b in pirate.boat_set.all()[:2]]
        ))
        self.assertEqual(pirate.boat_set.count(), 1)

    def test_urls(self):
        class AlphaResource(ResourceView):
            pass

        self.assertEqual(
            AlphaResource._meta.url_regex, 'alpha/(?P<alpha>[^/]+)?')

        class BetaResource(ResourceView):
            class Meta:
                parent = AlphaResource

        self.assertEqual(
            BetaResource._meta.url_regex,
            'alpha/(?P<alpha>[^/]+)?/beta/(?P<beta>[^/]+)?')

        class GammaResource(BetaResource):
            class Meta:
                prefix = 'gamma-prefix'

        self.assertEqual(
            GammaResource._meta.url_regex,
            'alpha/(?P<alpha>[^/]+)?/gamma-prefix/gamma/(?P<gamma>[^/]+)?')

        class ZetaResource(ResourceView):
            class Meta:
                parent = GammaResource

        self.assertEqual(
            ZetaResource._meta.url_regex,
            'alpha/(?P<alpha>[^/]+)?/gamma-prefix/gamma/(?P<gamma>[^/]+)?/zeta/(?P<zeta>[^/]+)?') # nolint

# lint_ignore=F0401,C0110



from django.db import models
from django.views.generic import View
from django.test import RequestFactory
from mixer.backend.django import mixer

from adrest.mixin import HandlerMixin
from ..api import api as API


class CoreHandlerTest(API.testCase):

    def test_meta_model(self):

        class Resource(View, HandlerMixin):

            class Meta:
                model = 'core.pirate'

        self.assertTrue(issubclass(Resource._meta.model, models.Model))

    def test_meta_name(self):

        class Resource(View, HandlerMixin):

            class Meta:
                model = 'core.pirate'

        self.assertEqual(Resource._meta.name, 'pirate')

        class IslandResource(View, HandlerMixin):

            class Meta:
                name = 'map'
                model = 'core.island'

        self.assertEqual(IslandResource._meta.name, 'map')

        class TreasureResource(View, HandlerMixin):

            class Meta:
                parent = Resource
                model = 'core.treasure'

        self.assertEqual(TreasureResource._meta.name, 'treasure')

    def test_allowed_methods(self):

        resource = self.api.resources.get('pirate')
        self.assertEqual(
            resource._meta.allowed_methods,
            ('GET', 'POST', 'PUT', 'DELETE', 'PATCH', 'OPTIONS', 'HEAD'))

    def test_methods(self):

        for _ in xrange(3):
            pirate = mixer.blend('core.pirate', character='good')

        response = self.get_resource('pirate')
        self.assertContains(response, '"count": 3')

        response = self.post_resource('pirate', data=dict(
            name='John',
            character='evil',
        ))
        self.assertContains(response, '"name": "John"')

        john = response.response
        response = self.put_resource('pirate', pirate=john, data=dict(
            name='Billy'
        ))
        self.assertContains(response, '"name": "Billy"')
        billy = response.response
        self.assertEqual(john, billy)

        response = self.delete_resource('pirate', pirate=billy)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(type(billy).objects.filter(name='Billy').count(), 0)

        response = self.patch_resource('pirate', data=dict(
            pirate=[1, 2],
            name='Tom'
        ))
        self.assertEqual(len(response.response), 2)

        for pirate in response.response:
            self.assertEqual(pirate.name, 'Tom')
            self.assertTrue(pirate.pk in [1, 2])

    def test_mixin(self):

        pirate = mixer.blend('core.pirate')

        class SomeResource(HandlerMixin, View):

            class Meta:
                allowed_methods = 'get', 'post'
                model = 'core.pirate'

            def dispatch(self, request, **resources):

                self.check_method_allowed(request)

                resources = self.get_resources(request, **resources)

                return self.handle_request(request, **resources)

        rf = RequestFactory()
        request = rf.get('/')
        resource = SomeResource()
        response = resource.dispatch(request)
        self.assertTrue(pirate in response.resources)

    def test_resources(self):
        pirates = mixer.cycle(2).blend('core.pirate', character='evil')
        response = self.put_resource(
            'pirate', data=dict(
                pirate=[p.pk for p in pirates],
                character='good',
            )
        )
        for p in response.json:
            self.assertEqual(p['fields']['character'], 'good')

        response = self.put_resource(
            'pirate', data=dict(
                pirate=[p.pk for p in pirates],
                character='sorrow',
            ), json=True
        )
        for p in response.json:
            self.assertEqual(p['fields']['character'], 'sorrow')


# lint_ignore=F0401,C,E1103



from django.test import TestCase


class BaseTestCase(TestCase):

    def test_response(self):
        response = self.client.get('/')
        self.assertEqual(response.status_code, 200)

        from django.contrib.auth.models import User

        user = User.objects.create(username='testuser')
        user.set_password('testpassword')
        user.save()

        self.client.login(username='testuser', password='testpassword')

        response = self.client.get('/')
        self.assertContains(response, 'logout')

        response = self.client.get('/logout', follow=True)
        self.assertNotContains(response, 'logout')



#!/usr/bin/env python
import os

from django.core.wsgi import get_wsgi_application


os.environ['DJANGO_SETTINGS_MODULE'] = os.environ.get(
    'DJANGO_SETTINGS_MODULE', 'settings.dev')
application = get_wsgi_application()

if 'dev' in os.environ['DJANGO_SETTINGS_MODULE']:
    from werkzeug.debug import DebuggedApplication
    from django.views import debug

    def null_technical_500_response(request, exc_type, exc_value, tb):
        raise exc_type, exc_value, tb
    debug.technical_500_response = null_technical_500_response
    application = DebuggedApplication(application, evalex=True)
